# -*- coding: utf-8 -*-
from plone.app.contentrules.testing import PLONE_APP_CONTENTRULES_FUNCTIONAL_TESTING  # noqa: E501
from plone.app.testing import applyProfile
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.contentrules.engine.interfaces import IRuleAssignmentManager
from plone.contentrules.engine.interfaces import IRuleStorage
from Products.GenericSetup.context import TarballExportContext
from Products.GenericSetup.interfaces import IBody
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.lifecycleevent.interfaces import IObjectModifiedEvent

import time
import unittest


class TestGenericSetup(unittest.TestCase):

    layer = PLONE_APP_CONTENTRULES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'news')
        self.portal.invokeFactory('Folder', 'events')
        self.storage = getUtility(IRuleStorage)
        applyProfile(self.portal, 'plone.app.contentrules:testing')

    def testRuleInstalled(self):
        self.assertTrue('test1' in self.storage)
        self.assertTrue('test2' in self.storage)

    def testRulesConfigured(self):
        rule1 = self.storage['test1']
        self.assertEqual('Test rule 1', rule1.title)
        self.assertEqual('A test rule', rule1.description)
        self.assertEqual(IObjectModifiedEvent, rule1.event)
        self.assertEqual(True, rule1.enabled)
        self.assertEqual(False, rule1.stop)

        self.assertEqual(2, len(rule1.conditions))
        self.assertEqual('plone.conditions.PortalType',
                         rule1.conditions[0].element)
        self.assertEqual(set(['Document', 'News Item']),
                         set(rule1.conditions[0].check_types))
        self.assertEqual('plone.conditions.Role', rule1.conditions[1].element)
        self.assertEqual(['Manager'], list(rule1.conditions[1].role_names))

        self.assertEqual(1, len(rule1.actions))
        self.assertEqual('plone.actions.Notify', rule1.actions[0].element)
        self.assertEqual(u'A message: Hej d\xe5', rule1.actions[0].message)
        self.assertEqual('info', rule1.actions[0].message_type)

        rule2 = self.storage['test2']
        self.assertEqual('Test rule 2', rule2.title)
        self.assertEqual('Another test rule', rule2.description)
        self.assertEqual(IObjectModifiedEvent, rule2.event)
        self.assertEqual(False, rule2.enabled)
        self.assertEqual(True, rule2.stop)

        self.assertEqual(1, len(rule2.conditions))
        self.assertEqual('plone.conditions.PortalType',
                         rule2.conditions[0].element)
        self.assertEqual(['Event'], list(rule2.conditions[0].check_types))

        self.assertEqual(1, len(rule2.actions))
        self.assertEqual('plone.actions.Workflow', rule2.actions[0].element)
        self.assertEqual('publish', rule2.actions[0].transition)

    def testRuleAssigned(self):
        assignable = IRuleAssignmentManager(self.portal.news)
        self.assertEqual(3, len(assignable))

        self.assertEqual(True, assignable['test1'].enabled)
        self.assertEqual(False, assignable['test1'].bubbles)

        self.assertEqual(False, assignable['test2'].enabled)
        self.assertEqual(True, assignable['test2'].bubbles)

        self.assertEqual(False, assignable['test3'].enabled)
        self.assertEqual(False, assignable['test3'].bubbles)

    def testAssignmentOrdering(self):
        assignable = IRuleAssignmentManager(self.portal.news)
        self.assertEqual(set([u'test3', u'test2', u'test1']), set(assignable.keys()))

    def testImportTwice(self):
        # Ensure rules, actions/conditions and assignments are not duplicated
        # if the profile is re-imported; see bug #8027.
        portal_setup = self.portal.portal_setup
        time.sleep(1)  # avoid timestamp colission
        portal_setup.runAllImportStepsFromProfile(
            'profile-plone.app.contentrules:testing')

        # We should get the same results as before
        self.testRuleInstalled()
        self.testRulesConfigured()
        self.testRuleAssigned()

    def testExport(self):
        self.maxDiff = None
        site = self.portal
        context = TarballExportContext(self.portal.portal_setup)
        exporter = getMultiAdapter(
            (site, context), IBody, name=u'plone.contentrules')

        expected = u"""<?xml version="1.0" encoding="utf-8"?>
<contentrules>
 <rule name="test1" title="Test rule 1" cascading="False"
    description="A test rule" enabled="True"
    event="zope.lifecycleevent.interfaces.IObjectModifiedEvent"
    stop-after="False">
  <conditions>
   <condition type="plone.conditions.PortalType">
    <property name="check_types">
     <element>Document</element>
     <element>News Item</element>
    </property>
   </condition>
   <condition type="plone.conditions.Role">
    <property name="role_names">
     <element>Manager</element>
    </property>
   </condition>
  </conditions>
  <actions>
   <action type="plone.actions.Notify">
    <property name="message">A message: Hej då</property>
    <property name="message_type">info</property>
   </action>
  </actions>
 </rule>
 <rule name="test2" title="Test rule 2" cascading="False"
    description="Another test rule" enabled="False"
    event="zope.lifecycleevent.interfaces.IObjectModifiedEvent"
    stop-after="True">
  <conditions>
   <condition type="plone.conditions.PortalType">
    <property name="check_types">
     <element>Event</element>
    </property>
   </condition>
  </conditions>
  <actions>
   <action type="plone.actions.Workflow">
    <property name="transition">publish</property>
   </action>
  </actions>
 </rule>
 <rule name="test3" title="Test rule 3" cascading="False"
    description="Third test rule" enabled="True"
    event="zope.lifecycleevent.interfaces.IObjectMovedEvent"
    stop-after="False">
  <conditions/>
  <actions/>
 </rule>
 <rule name="test4" title="Test rule 4" cascading="False"
    description="We move published events in a folder" enabled="True"
    event="Products.CMFCore.interfaces.IActionSucceededEvent"
    stop-after="True">
  <conditions>
   <condition type="plone.conditions.PortalType">
    <property name="check_types">
     <element>Event</element>
    </property>
   </condition>
   <condition type="plone.conditions.WorkflowTransition">
    <property name="wf_transitions">
     <element>publish</element>
    </property>
   </condition>
  </conditions>
  <actions>
   <action type="plone.actions.Move">
    <property name="target_folder">/events</property>
   </action>
  </actions>
 </rule>
 <rule name="test5" title="Test rule 5" cascading="True"
    description="Auto publish events" enabled="True"
    event="zope.lifecycleevent.interfaces.IObjectAddedEvent"
    stop-after="False">
  <conditions>
   <condition type="plone.conditions.PortalType">
    <property name="check_types">
     <element>Event</element>
    </property>
   </condition>
  </conditions>
  <actions>
   <action type="plone.actions.Workflow">
    <property name="transition">publish</property>
   </action>
  </actions>
 </rule>
 <assignment name="test4" bubbles="False" enabled="False" location=""/>
 <assignment name="test5" bubbles="False" enabled="False" location=""/>
 <assignment name="test1" bubbles="False" enabled="True" location="/news"/>
 <assignment name="test2" bubbles="True" enabled="False" location="/news"/>
 <assignment name="test3" bubbles="False" enabled="False" location="/news"/>
</contentrules>
"""

        body = exporter.body.decode('utf8')
        self.assertEqual(expected.strip(), body.strip(), body)



# -*- coding: utf-8 -*-
from plone.app.contentrules import handlers
from unittest import TestCase
from zope.lifecycleevent import ObjectAddedEvent
from zope.lifecycleevent import ObjectRemovedEvent


class TestModifyAction(TestCase):

    def setUp(self):
        self.called = False

        def register_call(testcase):

            def inner_register_call(event):
                testcase.called = True

            return inner_register_call

        self.original_execute_rules = handlers.execute_rules
        handlers.execute_rules = register_call(self)

    def tearDown(self):
        handlers.execute_rules = self.original_execute_rules
        self.called = False

    def testIgnoreAddedEvents(self):
        class Content(object):
            __parent__ = None
            __name__ = None
        handlers.modified(ObjectAddedEvent(Content()))
        self.assertFalse(self.called)

    def testIgnoreDeletedEvents(self):
        class Content(object):
            __parent__ = None
            __name__ = None
        handlers.modified(ObjectRemovedEvent(Content()))
        self.assertFalse(self.called)



# -*- coding: utf-8 -*-
from plone.app.contentrules.conditions.wftransition import WorkflowTransitionCondition  # noqa
from plone.app.contentrules.conditions.wftransition import WorkflowTransitionEditFormView  # noqa
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleCondition
from Products.CMFCore.interfaces import IActionSucceededEvent
from Products.CMFCore.WorkflowCore import ActionSucceededEvent
from zope.component import getMultiAdapter
from zope.component import getUtility


class TestWorkflowTransitionCondition(ContentRulesTestCase):

    def testRegistered(self):
        element = getUtility(
            IRuleCondition, name='plone.conditions.WorkflowTransition')
        self.assertEqual(
            'plone.conditions.WorkflowTransition', element.addview)
        self.assertEqual('edit', element.editview)
        self.assertEqual(None, element.for_)
        self.assertEqual(IActionSucceededEvent, element.event)

    def testInvokeAddView(self):
        element = getUtility(
            IRuleCondition, name='plone.conditions.WorkflowTransition')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter(
            (rule, self.portal.REQUEST), name='+condition')
        addview = getMultiAdapter(
            (adding, self.portal.REQUEST), name=element.addview)

        addview.form_instance.update()
        content = addview.form_instance.create(
            data={'wf_transitions': ['publish', 'hide']})
        addview.form_instance.add(content)

        e = rule.conditions[0]
        self.assertTrue(isinstance(e, WorkflowTransitionCondition))
        self.assertEqual(['publish', 'hide'], e.wf_transitions)

    def testInvokeEditView(self):
        element = getUtility(
            IRuleCondition, name='plone.conditions.WorkflowTransition')
        e = WorkflowTransitionCondition()
        editview = getMultiAdapter(
            (e, self.folder.REQUEST), name=element.editview)
        self.assertTrue(isinstance(editview, WorkflowTransitionEditFormView))

    def testExecute(self):
        e = WorkflowTransitionCondition()
        e.wf_transitions = ['publish', 'hide']

        ex = getMultiAdapter(
            (
                self.portal,
                e,
                ActionSucceededEvent(
                    self.folder,
                    'dummy_workflow',
                    'publish',
                    None,
                )
            ),
            IExecutable,
        )
        self.assertTrue(ex())

        ex = getMultiAdapter(
            (
                self.portal,
                e,
                ActionSucceededEvent(
                    self.folder,
                    'dummy_workflow',
                    'retract',
                    None,
                )
            ),
            IExecutable,
        )
        self.assertFalse(ex())

        ex = getMultiAdapter(
            (
                self.portal,
                e,
                ActionSucceededEvent(
                    self.folder,
                    'dummy_workflow',
                    'hide',
                    None,
                )
            ),
            IExecutable,
        )
        self.assertTrue(ex())



# -*- coding: utf-8 -*-
from AccessControl import Unauthorized
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.interfaces import IContentRulesControlPanel
from plone.app.contentrules.rule import get_assignments
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.memoize.instance import memoize
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory


def get_trigger_class(trigger):
    return 'trigger-{0}'.format(trigger.__identifier__.split('.')[-1].lower())


@implementer(IContentRulesControlPanel)
class ContentRulesControlPanel(BrowserView):
    """Manage rules in a the global rules container
    """
    template = ViewPageTemplateFile('templates/controlpanel.pt')

    def __call__(self):
        form = self.request.form
        if form.get('rule-id', False):
            if form.get('form.button.EnableRule', None) is not None:
                self.enable_rule()
            elif form.get('form.button.DisableRule', None) is not None:
                self.disable_rule()
            elif form.get('form.button.DeleteRule', None) is not None:
                self.delete_rule()

        if form.get('global_disable', None) is not None:
            if form['global_disable']:
                msg = self.globally_disable()
            else:
                msg = self.globally_enable()

            IStatusMessage(self.request).add(msg)

        return self.template()

    def authorize(self):
        authenticator = getMultiAdapter((self.context, self.request),
                                        name=u'authenticator')
        if not authenticator.verify():
            raise Unauthorized

    def globally_disabled(self):
        storage = getUtility(IRuleStorage)
        return not storage.active

    def registeredRules(self):
        rules = self._getRules()

        events = dict([(e.value, e.token) for e in self._events()])
        info = []
        for r in rules:
            trigger_class = get_trigger_class(r.event)
            enabled_class = r.enabled and 'state-enabled' or 'state-disabled'
            assigned = len(get_assignments(r)) > 0

            assigned_class = 'assignment-unassigned'
            if assigned:
                assigned_class = 'assignment-assigned'

            info.append({
                'id': r.__name__,
                'title': r.title,
                'description': r.description,
                'enabled': r.enabled,
                'assigned': assigned,
                'trigger': events[r.event],
                'row_class': '{0} {1} {2}'.format(
                    trigger_class,
                    enabled_class,
                    assigned_class,
                )
            })

        return info

    def ruleTypesToShow(self):
        selector = []
        rules = self._getRules()
        for event in self._events():
            # filter unused rule types
            for rule in rules:
                if rule.event == event.value:
                    break
            else:
                continue

            eventname = translate(
                event.token, context=self.request, domain='plone')
            selector.append({'id': get_trigger_class(event.value),
                             'title': eventname})

        return selector

    def statesToShow(self):
        return (
            {
                'id': 'state-enabled',
                'title': _(u'label_rule_enabled', default=u'Enabled'),
            },
            {
                'id': 'state-disabled',
                'title': _(u'label_rule_disabled', default=u'Disabled'),
            },
        )

    def _getRules(self):
        storage = getUtility(IRuleStorage)
        return storage.values()

    @memoize
    def _events(self):
        eventsFactory = getUtility(
            IVocabularyFactory, name='plone.contentrules.events')
        return eventsFactory(self.context)

    def delete_rule(self):
        self.authorize()
        rule_id = self.request['rule-id']
        storage = getUtility(IRuleStorage)
        del storage[rule_id]
        return 'ok'

    def enable_rule(self):
        self.authorize()
        storage = getUtility(IRuleStorage)
        rule_id = self.request['rule-id']
        storage[rule_id].enabled = True
        return 'ok'

    def disable_rule(self):
        self.authorize()
        storage = getUtility(IRuleStorage)
        rule_id = self.request['rule-id']
        storage[rule_id].enabled = False
        return 'ok'

    def globally_disable(self):
        self.authorize()
        storage = getUtility(IRuleStorage)
        storage.active = False
        return translate(_('Content rules has been globally disabled'),
                         context=self.request)

    def globally_enable(self):
        self.authorize()
        storage = getUtility(IRuleStorage)
        storage.active = True
        return translate(_('Content rules has been globally enabled'),
                         context=self.request)



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from OFS.CopySupport import sanity_check
from OFS.event import ObjectWillBeMovedEvent
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.actions import ActionAddForm
from plone.app.contentrules.actions import ActionEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.vocabularies.catalog import CatalogSource
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import utils
from Products.statusmessages.interfaces import IStatusMessage
from ZODB.POSException import ConflictError
from zope import schema
from zope.component import adapter
from zope.container.contained import notifyContainerModified
from zope.event import notify
from zope.interface import implementer
from zope.interface import Interface
from zope.lifecycleevent import ObjectMovedEvent


class IMoveAction(Interface):
    """Interface for the configurable aspects of a move action.

    This is also used to create add and edit forms, below.
    """

    target_folder = schema.Choice(
        title=_(u'Target folder'),
        description=_(u'As a path relative to the portal root.'),
        required=True,
        source=CatalogSource(is_folderish=True),
    )


@implementer(IMoveAction, IRuleElementData)
class MoveAction(SimpleItem):
    """The actual persistent implementation of the action element.
    """

    target_folder = ''
    element = 'plone.actions.Move'

    @property
    def summary(self):
        return _(
            u'Move to folder ${folder}',
            mapping=dict(folder=self.target_folder)
        )


@adapter(Interface, IMoveAction, Interface)
@implementer(IExecutable)
class MoveActionExecutor(object):
    """The executor for this action.
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        portal_url = getToolByName(self.context, 'portal_url', None)
        if portal_url is None:
            return False

        obj = self.event.object
        parent = aq_parent(aq_inner(obj))

        path = self.element.target_folder
        if len(path) > 1 and path[0] == '/':
            path = path[1:]
        target = portal_url.getPortalObject().unrestrictedTraverse(
            str(path),
            None,
        )

        if target is None:
            self.error(
                obj,
                _(
                    u'Target folder ${target} does not exist.',
                    mapping={'target': path},
                )
            )
            return False

        if target.absolute_url() == parent.absolute_url():
            # We're already here!
            return True

        try:
            obj._notifyOfCopyTo(target, op=1)
        except ConflictError:
            raise
        except Exception as e:
            self.error(obj, str(e))
            return False

        # Are we trying to move into the same container that we copied from?
        if not sanity_check(target, obj):
            return False

        old_id = obj.getId()
        new_id = self.generate_id(target, old_id)

        notify(ObjectWillBeMovedEvent(obj, parent, old_id, target, new_id))

        obj.manage_changeOwnershipType(explicit=1)

        parent._delObject(old_id, suppress_events=True)
        obj = aq_base(obj)
        obj._setId(new_id)

        target._setObject(new_id, obj, set_owner=0, suppress_events=True)
        obj = target._getOb(new_id)

        notify(ObjectMovedEvent(obj, parent, old_id, target, new_id))
        notifyContainerModified(parent)
        if aq_base(parent) is not aq_base(target):
            notifyContainerModified(target)

        obj._postCopy(target, op=1)

        # try to make ownership implicit if possible
        obj.manage_changeOwnershipType(explicit=0)

        return True

    def error(self, obj, error):
        request = getattr(self.context, 'REQUEST', None)
        if request is not None:
            title = utils.pretty_title_or_id(obj, obj)
            message = _(
                u'Unable to move ${name} as part of content rule '
                u"'move' action: ${error}",
                mapping={'name': title, 'error': error}
            )
            IStatusMessage(request).addStatusMessage(message, type='error')

    def generate_id(self, target, old_id):
        taken = getattr(aq_base(target), 'has_key', None)
        if taken is None:
            item_ids = set(target.objectIds())

            def taken(x): return x in item_ids
        if not taken(old_id):
            return old_id
        idx = 1
        while taken('{0}.{1}'.format(old_id, idx)):
            idx += 1
        return '{0}.{1}'.format(old_id, idx)


class MoveAddForm(ActionAddForm):
    """An add form for move-to-folder actions.
    """
    schema = IMoveAction
    label = _(u'Add Move Action')
    description = _(u'A move action can move an object to a different folder.')
    form_name = _(u'Configure element')
    Type = MoveAction


class MoveAddFormView(ContentRuleFormWrapper):
    form = MoveAddForm


class MoveEditForm(ActionEditForm):
    """An edit form for move rule actions.

    z3c.form does all the magic here.
    """
    schema = IMoveAction
    label = _(u'Edit Move Action')
    description = _(u'A move action can move an object to a different folder.')
    form_name = _(u'Configure element')


class MoveEditFormView(ContentRuleFormWrapper):
    form = MoveEditForm



# -*- coding: utf-8 -*-
from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import EditForm
from plone.app.uuid.utils import uuidToPhysicalPath
from plone.uuid.interfaces import IUUID
from z3c.form import form
from zope.component.hooks import getSite

import os


class ContentWrapper(object):
    """
    The sole purpose of this is to transform target_folder
    values from UUID to path, which all of content rules expects
    """

    def __init__(self, content):
        self.__dict__['content'] = content

    @property
    def target_folder(self):
        content = self.content

        if content.target_folder and content.target_folder[0] == '/':
            # need to convert to uuid
            site = getSite()
            site_path = '/'.join(site.getPhysicalPath())
            path = os.path.join(site_path, content.target_folder.lstrip('/'))
            target = site.restrictedTraverse(path, None)
            if target is not None:
                return IUUID(target, None)

    def __getattr__(self, name, default=None):
        return getattr(self.__dict__['content'], name, default)

    def __setattr__(self, name, value):
        setattr(self.__dict__['content'], name, value)


class ActionAddForm(AddForm):
    Type = None

    def create(self, data):
        """
        Since content rules expects paths, we're transforming UUID, which
        is what the z3c form widget uses, to paths.
        """
        a = self.Type()
        if data.get('target_folder', None):
            site = getSite()
            site_path = '/'.join(site.getPhysicalPath())
            path = uuidToPhysicalPath(data['target_folder'])
            if path:
                data['target_folder'] = path[len(site_path):]
        form.applyChanges(self, a, data)
        return a


class ActionEditForm(EditForm):

    def getContent(self):
        return ContentWrapper(super(ActionEditForm, self).getContent())



# -*- coding: utf-8 -*-
from plone.app.contentrules.actions.delete import DeleteAction
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleAction
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.interfaces import IObjectEvent
from zope.interface import implementer
from plone.app.testing import login
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import setRoles


@implementer(IObjectEvent)
class DummyEvent(object):

    def __init__(self, object):
        self.object = object


class TestDeleteAction(ContentRulesTestCase):

    def testRegistered(self):
        element = getUtility(IRuleAction, name='plone.actions.Delete')
        self.assertEqual('plone.actions.Delete', element.addview)
        self.assertEqual(None, element.editview)
        self.assertEqual(None, element.for_)
        self.assertEqual(IObjectEvent, element.event)

    def testInvokeAddView(self):
        element = getUtility(IRuleAction, name='plone.actions.Delete')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter(
            (rule.__of__(self.portal), self.portal.REQUEST), name='+action')
        addview = getMultiAdapter(
            (adding.__of__(rule), self.portal.REQUEST), name=element.addview)
        addview()

        e = rule.actions[0]
        self.assertTrue(isinstance(e, DeleteAction))

    def testExecute(self):
        e = DeleteAction()

        ex = getMultiAdapter(
            (self.folder, e, DummyEvent(self.folder.d1)), IExecutable)
        self.assertEqual(True, ex())

        self.assertFalse('d1' in self.folder.objectIds())



# -*- coding: utf-8 -*-
from zope.browser.interfaces import IAdding
from zope.interface import Interface


class IContentRulesInfo(Interface):
    """Site-wide information about content rules
    """

    def show_rules_tab():
        """Determine whether or not the rules tab should be shown.
        """


class IContentRulesControlPanel(Interface):
    """Marker interface for rules control panel view
    """

    def globally_disabled():
        """Wether content rules are globally disabled or not"""


class IRuleAdding(IAdding):
    """Marker interface for rule add views.

    Rules' addviews should be registered for this.
    """


class IRuleElementAdding(IAdding):
    """Marker interface for rule element (actions/conditions) add views.

    Rules' addviews should be registered for this.
    """


class IRuleConditionAdding(IRuleElementAdding):
    pass


class IRuleActionAdding(IRuleElementAdding):
    pass


class IContentRulesForm(Interface):
    """Marker interface for forms that need content rules layout
    """



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from Acquisition import aq_inner
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.contentrules.browser.formhelper import EditForm
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.interfaces import ITypesTool
from Products.CMFCore.utils import getToolByName
from z3c.form import form
from zope import schema
from zope.component import adapter
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface
from zope.site.hooks import getSite


class IPortalTypeCondition(Interface):
    """Interface for the configurable aspects of a portal type condition.

    This is also used to create add and edit forms, below.
    """

    check_types = schema.Set(
        title=_(u'Content type'),
        description=_(u'The content type to check for.'),
        required=True,
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.ReallyUserFriendlyTypes'
        )
    )


@implementer(IPortalTypeCondition, IRuleElementData)
class PortalTypeCondition(SimpleItem):
    """The actual persistent implementation of the portal type condition element.

    Note that we must mix in SimpleItem to keep Zope 2 security happy.
    """

    check_types = []
    element = 'plone.conditions.PortalType'

    @property
    def summary(self):
        portal = getSite()
        portal_types = getToolByName(portal, 'portal_types')
        titles = []
        for name in self.check_types:
            fti = getattr(portal_types, name, None)
            if fti is not None:
                title = translate(fti.Title(), context=portal.REQUEST)
                titles.append(title)
        return _(
            u'Content types are: ${names}',
            mapping=dict(names=', '.join(titles))
        )


@implementer(IExecutable)
@adapter(Interface, IPortalTypeCondition, Interface)
class PortalTypeConditionExecutor(object):
    """The executor for this condition.

    This is registered as an adapter in configure.zcml
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        obj = aq_inner(self.event.object)
        if not hasattr(aq_base(obj), 'getTypeInfo'):
            return False
        elif ITypesTool.providedBy(obj):
            # types tool have a getTypeInfo method
            return False

        ti = obj.getTypeInfo()  # getTypeInfo can be None
        if ti is None:
            return False
        return ti.getId() in self.element.check_types


class PortalTypeAddForm(AddForm):
    """An add form for portal type conditions.
    """
    schema = IPortalTypeCondition
    label = _(u'Add Content Type Condition')
    description = _(
        u'A portal type condition makes the rule apply only to '
        u'certain content types.'
    )
    form_name = _(u'Configure element')

    def create(self, data):
        c = PortalTypeCondition()
        form.applyChanges(self, c, data)
        return c


class PortalTypeAddFormView(ContentRuleFormWrapper):
    form = PortalTypeAddForm


class PortalTypeEditForm(EditForm):
    """An edit form for portal type conditions
    """
    schema = IPortalTypeCondition
    label = _(u'Edit Content Type Condition')
    description = _(
        u'A portal type condition makes the rule apply only to certain '
        u'content types.'
    )
    form_name = _(u'Configure element')


class PortalTypeEditFormView(ContentRuleFormWrapper):
    form = PortalTypeEditForm



# -*- coding: utf-8 -*-
from plone.app.contentrules.rule import get_assignments
from plone.app.contentrules.rule import insert_assignment
from plone.contentrules.engine.assignments import RuleAssignment
from plone.contentrules.engine.interfaces import IRuleAssignmentManager
from plone.contentrules.engine.interfaces import IRuleStorage
from zope.component import queryUtility


def assign_rule(container, rule_id, enabled=True, bubbles=True,
                insert_before=None):
    """Assign
       @param string rule_id
       rule to
       @param object container
    with options
       @param bool enabled
       @param bool bubbles (apply in subfolders)
       @param string insert-before
    """
    storage = queryUtility(IRuleStorage)
    if storage is None:
        return

    assignable = IRuleAssignmentManager(container, None)
    if assignable is None:
        return

    assignment = assignable.get(rule_id, None)
    if assignment is None:
        assignable[rule_id] = RuleAssignment(rule_id)

    assignable[rule_id].enabled = bool(enabled)
    assignable[rule_id].bubbles = bool(bubbles)
    path = '/'.join(container.getPhysicalPath())
    insert_assignment(storage[rule_id], path)

    if insert_before:
        position = None
        keys = list(assignable.keys())
        if insert_before == '*':
            position = 0
        elif insert_before in keys:
            position = keys.index(insert_before)

        if position is not None:
            keys.remove(rule_id)
            keys.insert(position, rule_id)
            assignable.updateOrder(keys)


def unassign_rule(container, rule_id):
    """Remove
       @param string rule_id
       rule from
       @param object container
    """
    assignable = IRuleAssignmentManager(container)
    storage = queryUtility(IRuleStorage)
    path = '/'.join(container.getPhysicalPath())
    del assignable[rule_id]
    get_assignments(storage[rule_id]).remove(path)


def edit_rule_assignment(container, rule_id, bubbles=None, enabled=None):
    """Change a property of an assigned rule
    @param object container
    @param string rule_id
    @param bool enabled
    @param bool bubbles (apply in subfolders)
    """
    assignable = IRuleAssignmentManager(container)
    assignment = assignable.get(rule_id, None)
    if bubbles is not None:
        assignment.bubbles = bool(bubbles)

    if enabled is not None:
        assignment.enabled = bool(enabled)



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.actions import ActionAddForm
from plone.app.contentrules.actions import ActionEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.statusmessages.interfaces import IStatusMessage
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class INotifyAction(Interface):
    """Interface for the configurable aspects of a notify action.

    This is also used to create add and edit forms, below.
    """

    message = schema.TextLine(
        title=_(u'Message'),
        description=_(u'The message to send to the user.'),
        required=True,
    )

    message_type = schema.Choice(
        title=_(u'Message type'),
        description=_(u'Select the type of message to display.'),
        values=('info', 'warning', 'error'),
        required=True,
        default='info',
    )


@implementer(INotifyAction, IRuleElementData)
class NotifyAction(SimpleItem):
    """The actual persistent implementation of the notify action element.
    """

    message = ''
    message_type = ''

    element = 'plone.actions.Notify'

    @property
    def summary(self):
        return _(
            u'Notify with message ${message}',
            mapping=dict(message=self.message),
        )


@adapter(Interface, INotifyAction, Interface)
@implementer(IExecutable)
class NotifyActionExecutor(object):
    """The executor for this action.

    This is registered as an adapter in configure.zcml
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        request = self.context.REQUEST
        message = _(self.element.message)
        message_type = self.element.message_type
        IStatusMessage(request).addStatusMessage(message, type=message_type)
        return True


class NotifyAddForm(ActionAddForm):
    """An add form for notify rule actions.
    """
    schema = INotifyAction
    label = _(u'Add Notify Action')
    description = _(u'A notify action can show a message to the user.')
    form_name = _(u'Configure element')
    Type = NotifyAction


class NotifyAddFormView(ContentRuleFormWrapper):
    form = NotifyAddForm


class NotifyEditForm(ActionEditForm):
    """An edit form for notify rule actions.

    z3c.form does all the magic here.
    """
    schema = INotifyAction
    label = _(u'Edit Notify Action')
    description = _(u'A notify action can show a message to the user.')
    form_name = _(u'Configure element')


class NotifyEditFormView(ContentRuleFormWrapper):
    form = NotifyAddForm



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.actions import ActionAddForm
from plone.app.contentrules.actions import ActionEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from plone.registry.interfaces import IRegistry
from plone.stringinterp.interfaces import IStringInterpolator
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.MailHost.MailHost import MailHostError
from Products.statusmessages.interfaces import IStatusMessage
from smtplib import SMTPException
from zope import schema
from zope.component import adapter
from zope.component import getUtility
from zope.component.interfaces import ComponentLookupError
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.interface import Interface

import logging
import six


logger = logging.getLogger('plone.contentrules')


class IMailAction(Interface):
    """Definition of the configuration available for a mail action
    """
    subject = schema.TextLine(
        title=_(u'Subject'),
        description=_(u'Subject of the message'),
        required=True
    )
    source = schema.TextLine(
        title=_(u'Email source'),
        description=_(
            'The email address that sends the email. If no email is provided '
            'here, it will use the portal from address.'
        ),
        required=False
    )
    recipients = schema.TextLine(
        title=_(u'Email recipients'),
        description=_(
            'The email where you want to send this message. To send it to '
            'different email addresses, just separate them with ,'
        ),
        required=True
    )
    exclude_actor = schema.Bool(
        title=_(u'Exclude actor from recipients'),
        description=_('Do not send the email to the user that did the action.')
    )
    message = schema.Text(
        title=_(u'Message'),
        description=_(u'The message that you want to mail.'),
        required=True
    )


@implementer(IMailAction, IRuleElementData)
class MailAction(SimpleItem):
    """
    The implementation of the action defined before
    """

    subject = u''
    source = u''
    recipients = u''
    message = u''
    exclude_actor = False

    element = 'plone.actions.Mail'

    @property
    def summary(self):
        return _(u'Email report to ${recipients}',
                 mapping=dict(recipients=self.recipients))


@implementer(IExecutable)
@adapter(Interface, IMailAction, Interface)
class MailActionExecutor(object):
    """The executor for this action.
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event
        registry = getUtility(IRegistry)
        self.mail_settings = registry.forInterface(IMailSchema,
                                                   prefix='plone')

    def __call__(self):
        mailhost = getToolByName(aq_inner(self.context), 'MailHost')
        if not mailhost:
            raise ComponentLookupError(
                'You must have a Mailhost utility to execute this action'
            )

        email_charset = self.mail_settings.email_charset
        obj = self.event.object
        interpolator = IStringInterpolator(obj)
        source = self.element.source

        if source:
            source = interpolator(source).strip()

        if not source:
            # no source provided, looking for the site wide from email
            # address
            from_address = self.mail_settings.email_from_address
            if not from_address:
                # the mail can't be sent. Try to inform the user
                request = getRequest()
                if request:
                    messages = IStatusMessage(request)
                    msg = _(
                        u'Error sending email from content rule. You must '
                        u'provide a source address for mail '
                        u'actions or enter an email in the portal properties'
                    )
                    messages.add(msg, type=u'error')
                return False

            from_name = self.mail_settings.email_from_name.strip('"')
            if six.PY2 and isinstance(from_name, six.text_type):
                from_name = from_name.encode('utf8')
            source = '"{0}" <{1}>'.format(from_name, from_address)

        recip_string = interpolator(self.element.recipients)
        if recip_string:  # check recipient is not None or empty string
            recipients = set([
                str(mail.strip()) for mail in recip_string.split(',')
                if mail.strip()
            ])
        else:
            recipients = set()

        if self.element.exclude_actor:
            mtool = getToolByName(aq_inner(self.context), 'portal_membership')
            actor_email = mtool.getAuthenticatedMember().getProperty(
                'email',
                ''
            )
            if actor_email in recipients:
                recipients.remove(actor_email)

        # prepend interpolated message with \n to avoid interpretation
        # of first line as header
        message = u'\n{0}'.format(interpolator(self.element.message))

        subject = interpolator(self.element.subject)

        for email_recipient in recipients:
            try:
                # XXX: We're using "immediate=True" because otherwise we won't
                # be able to catch SMTPException as the smtp connection is made
                # as part of the transaction apparatus.
                # AlecM thinks this wouldn't be a problem if mail queuing was
                # always on -- but it isn't. (stevem)
                # so we test if queue is not on to set immediate
                mailhost.send(message, email_recipient, source,
                              subject=subject, charset=email_charset,
                              immediate=not mailhost.smtp_queue)
            except (MailHostError, SMTPException):
                logger.exception(
                    'mail error: Attempt to send mail in content rule failed'
                )

        return True


class MailAddForm(ActionAddForm):
    """
    An add form for the mail action
    """
    schema = IMailAction
    label = _(u'Add Mail Action')
    description = _(u'A mail action can mail different recipient.')
    form_name = _(u'Configure element')
    Type = MailAction
    # custom template will allow us to add help text
    template = ViewPageTemplateFile('templates/mail.pt')


class MailAddFormView(ContentRuleFormWrapper):
    form = MailAddForm


class MailEditForm(ActionEditForm):
    """
    An edit form for the mail action
    """
    schema = IMailAction
    label = _(u'Edit Mail Action')
    description = _(u'A mail action can mail different recipient.')
    form_name = _(u'Configure element')

    # custom template will allow us to add help text
    template = ViewPageTemplateFile('templates/mail.pt')


class MailEditFormView(ContentRuleFormWrapper):
    form = MailEditForm



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.interfaces import IRuleActionAdding
from plone.app.contentrules.browser.interfaces import IRuleAdding
from plone.app.contentrules.browser.interfaces import IRuleConditionAdding
from plone.contentrules.engine.interfaces import IRuleStorage
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from warnings import warn
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.container.interfaces import INameChooser
from zope.interface import implementer


@implementer(IRuleAdding)
class RuleAdding(SimpleItem, BrowserView):

    context = None
    request = None
    contentName = None

    # This is necessary so that context.absolute_url() works properly on the
    # add form, which in turn fixes the <base /> URL
    id = '+rule'

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def add(self, content):
        """Add the rule to the context
        """
        storage = getUtility(IRuleStorage)
        chooser = INameChooser(storage)
        name = chooser.chooseName(None, content)
        self._chosen_name = name
        storage[name] = content
        IStatusMessage(self.request).add(_(
            u'New content rule created. '
            u'Please add conditions and actions at the bottom of the page.'),
            type=u'info')

    def renderAddButton(self):
        warn('The renderAddButton method is deprecated, use nameAllowed',
             DeprecationWarning, 2)

    def namesAccepted(self):
        return False

    def nameAllowed(self):
        return False

    def isSingleMenuItem(self):
        return False

    def addingInfo(self):
        return []

    def hasCustomAddView(self):
        return None

    def nextURL(self):
        return None


class RuleElementAdding(SimpleItem, BrowserView):

    context = None
    request = None
    contentName = None

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def nextURL(self):
        url = str(getMultiAdapter(
            (aq_parent(self.context), self.request), name=u'absolute_url'))
        return url + '/@@manage-content-rules'

    def renderAddButton(self):
        warn('The renderAddButton method is deprecated, use nameAllowed',
             DeprecationWarning, 2)

    def namesAccepted(self):
        return False

    def nameAllowed(self):
        return False

    def isSingleMenuItem(self):
        return False

    def addingInfo(self):
        return []

    def hasCustomAddView(self):
        return None


@implementer(IRuleConditionAdding)
class RuleConditionAdding(RuleElementAdding):

    # This is necessary so that context.absolute_url() works properly on the
    # add form, which in turn fixes the <base /> URL
    id = '+condition'

    def add(self, content):
        """Add the rule element to the context rule
        """
        rule = aq_base(aq_inner(self.context))
        rule.conditions.append(content)


@implementer(IRuleActionAdding)
class RuleActionAdding(RuleElementAdding):

    # This is necessary so that context.absolute_url() works properly on the
    # add form, which in turn fixes the <base /> URL
    id = '+action'

    def add(self, content):
        """Add the rule element to the context rule
        """
        rule = aq_base(aq_inner(self.context))
        rule.actions.append(content)



# -*- coding: utf-8 -*-
from plone.app.contentrules.actions.workflow import WorkflowAction
from plone.app.contentrules.actions.workflow import WorkflowEditFormView
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleAction
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.interfaces import IObjectEvent
from zope.interface import implementer


@implementer(IObjectEvent)
class DummyEvent(object):

    def __init__(self, object):
        self.object = object


class TestWorkflowAction(ContentRulesTestCase):

    def testRegistered(self):
        element = getUtility(IRuleAction, name='plone.actions.Workflow')
        self.assertEqual('plone.actions.Workflow', element.addview)
        self.assertEqual('edit', element.editview)
        self.assertEqual(None, element.for_)
        self.assertEqual(IObjectEvent, element.event)

    def testInvokeAddView(self):
        element = getUtility(IRuleAction, name='plone.actions.Workflow')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter((rule, self.portal.REQUEST), name='+action')
        addview = getMultiAdapter(
            (adding, self.portal.REQUEST), name=element.addview)

        addview.form_instance.update()
        content = addview.form_instance.create(
            data={'transition': 'publish', })
        addview.form_instance.add(content)

        e = rule.actions[0]
        self.assertTrue(isinstance(e, WorkflowAction))
        self.assertEqual('publish', e.transition)

    def testInvokeEditView(self):
        element = getUtility(IRuleAction, name='plone.actions.Workflow')
        e = WorkflowAction()
        editview = getMultiAdapter(
            (e, self.folder.REQUEST), name=element.editview)
        self.assertTrue(isinstance(editview, WorkflowEditFormView))

    def testExecute(self):
        e = WorkflowAction()
        e.transition = 'publish'

        ex = getMultiAdapter(
            (self.folder, e, DummyEvent(self.folder.d1)), IExecutable)
        self.assertEqual(True, ex())

        self.assertEqual(
            'published',
            self.portal.portal_workflow.getInfoFor(
                self.folder.d1,
                'review_state',
            )
        )

    def testExecuteWithError(self):
        e = WorkflowAction()
        e.transition = 'foobar'

        old_state = self.portal.portal_workflow.getInfoFor(
            self.folder.d1, 'review_state')

        ex = getMultiAdapter(
            (self.folder, e, DummyEvent(self.folder.d1)), IExecutable)
        self.assertEqual(False, ex())

        self.assertEqual(
            old_state,
            self.portal.portal_workflow.getInfoFor(
                self.folder.d1,
                'review_state',
            )
        )



# -*- coding: utf-8 -*-
from plone.app.contentrules.actions.notify import NotifyAction
from plone.app.contentrules.actions.notify import NotifyEditFormView
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleAction
from Products.statusmessages import STATUSMESSAGEKEY
from Products.statusmessages.adapter import _decodeCookieValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import implementer
from zope.interface import Interface


@implementer(Interface)
class DummyEvent(object):
    pass


class TestNotifyAction(ContentRulesTestCase):

    def testRegistered(self):
        element = getUtility(IRuleAction, name='plone.actions.Notify')
        self.assertEqual('plone.actions.Notify', element.addview)
        self.assertEqual('edit', element.editview)
        self.assertEqual(None, element.for_)
        self.assertEqual(None, element.event)

    def testInvokeAddView(self):
        element = getUtility(IRuleAction, name='plone.actions.Notify')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter((rule, self.request), name='+action')
        addview = getMultiAdapter((adding, self.request), name=element.addview)

        addview.form_instance.update()
        content = addview.form_instance.create(data={'message': 'Hello world',
                                                     'message_type': 'info'})
        addview.form_instance.add(content)

        e = rule.actions[0]
        self.assertTrue(isinstance(e, NotifyAction))
        self.assertEqual('Hello world', e.message)
        self.assertEqual('info', e.message_type)

    def testInvokeEditView(self):
        element = getUtility(IRuleAction, name='plone.actions.Notify')
        e = NotifyAction()
        editview = getMultiAdapter((e, self.request), name=element.editview)
        self.assertTrue(isinstance(editview, NotifyEditFormView))

    def testExecute(self):
        e = NotifyAction()
        e.message = 'Hello world'
        e.message_type = 'info'

        ex = getMultiAdapter((self.folder, e, DummyEvent()), IExecutable)
        self.assertEqual(True, ex())

        new_cookies = self.request.RESPONSE.cookies[STATUSMESSAGEKEY]
        messages = _decodeCookieValue(new_cookies['value'])
        self.assertEqual(1, len(messages))
        self.assertEqual('Hello world', messages[0].message)
        self.assertEqual('info', messages[0].type)



# -*- coding: utf-8 -*-
import doctest


# Standard options for DocTests
optionflags = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE |
               doctest.REPORT_ONLY_FIRST_FAILURE)



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.contentrules.browser.formhelper import EditForm
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.Expression import createExprContext
from Products.CMFCore.Expression import Expression
from Products.CMFCore.utils import getToolByName
from z3c.form import form
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class ITalesExpressionCondition(Interface):
    """Interface for the configurable aspects of a TALES expression condition.

    This is also used to create add and edit forms, below.
    """

    tales_expression = schema.TextLine(
        title=_(u'TALES expression'),
        description=_(u'The TALES expression to check.'),
        required=True)


@implementer(ITalesExpressionCondition, IRuleElementData)
class TalesExpressionCondition(SimpleItem):
    """The actual persistent implementation of the TALES expression condition
    element.
    """

    tales_expression = ''
    element = 'plone.conditions.TalesExpression'

    @property
    def summary(self):
        return _(u'TALES expression is: ${tales_expression}',
                 mapping={'tales_expression': self.tales_expression})


@implementer(IExecutable)
@adapter(Interface, ITalesExpressionCondition, Interface)
class TalesExpressionConditionExecutor(object):
    """The executor for this condition.

    This is registered as an adapter in configure.zcml
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        object = self.event.object
        folder = self.context
        portal = getToolByName(folder, 'portal_url').getPortalObject()
        expression = self.element.tales_expression
        ec = createExprContext(folder, portal, object)
        # Workaround CMFCore/PageTemplates issue with unicode missing context
        ec.contexts['context'] = ec.contexts['here']
        return bool(Expression(expression)(ec))


class TalesExpressionAddForm(AddForm):
    """An add form for tales expression condition.
    """
    schema = ITalesExpressionCondition
    label = _(u'Add TALES Expression Condition')
    description = _(u'A TALES expression condition makes the rule apply '
                    u'only if TALES expression is not False in context.')
    form_name = _(u'Configure element')

    def create(self, data):
        c = TalesExpressionCondition()
        form.applyChanges(self, c, data)
        return c


class TalesExpressionAddFormView(ContentRuleFormWrapper):
    form = TalesExpressionAddForm


class TalesExpressionEditForm(EditForm):
    """An edit form for TALES expression condition
    """
    schema = ITalesExpressionCondition
    label = _(u'Edit TALES Expression Condition')
    description = _(u'A TALES expression condition makes the rule apply '
                    u'only if TALES expression is not False in context.')
    form_name = _(u'Configure element')


class TalesExpressionEditFormView(ContentRuleFormWrapper):
    form = TalesExpressionEditForm



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.contentrules.browser.formhelper import EditForm
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.utils import getToolByName
from z3c.form import form
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class IGroupCondition(Interface):
    """Interface for the configurable aspects of a group condition.

    This is also used to create add and edit forms, below.
    """

    group_names = schema.Set(
        title=_(u'Group name'),
        description=_(u'The name of the group.'),
        required=True,
        value_type=schema.Choice(vocabulary='plone.app.vocabularies.Groups')
    )


@implementer(IGroupCondition, IRuleElementData)
class GroupCondition(SimpleItem):
    """The actual persistent implementation of the group condition element.

    Note that we must mix in SimpleItem to keep Zope 2 security happy.
    """

    group_names = []
    element = 'plone.conditions.Group'

    @property
    def summary(self):
        return _(
            u'Groups are: ${names}',
            mapping=dict(names=', '.join(self.group_names))
        )


@implementer(IExecutable)
@adapter(Interface, IGroupCondition, Interface)
class GroupConditionExecutor(object):
    """The executor for this condition.

    This is registered as an adapter in configure.zcml
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        portal_membership = getToolByName(
            self.context, 'portal_membership', None)
        portal_groups = getToolByName(self.context, 'portal_groups', None)
        if portal_groups is None or portal_groups is None:
            return False
        member = portal_membership.getAuthenticatedMember()
        groupIds = [g.getId()
                    for g in portal_groups.getGroupsByUserId(member.getId())]
        for g in self.element.group_names:
            if g in groupIds:
                return True
        return False


class GroupAddForm(AddForm):
    """An add form for group rule conditions.
    """
    schema = IGroupCondition
    label = _(u'Add Group Condition')
    description = _(
        u'A group condition can prevent a rule from executing '
        u'unless the current user is a member of a particular group.'
    )
    form_name = _(u'Configure element')

    def create(self, data):
        c = GroupCondition()
        form.applyChanges(self, c, data)
        return c


class GroupAddFormView(ContentRuleFormWrapper):
    form = GroupAddForm


class GroupEditForm(EditForm):
    """An edit form for group conditions
    """
    schema = IGroupCondition
    label = _(u'Edit Group Condition')
    description = _(
        u'A group condition can prevent a rule from executing '
        u'unless the current user is a member of a particular group.'
    )
    form_name = _(u'Configure element')


class GroupEditFormView(ContentRuleFormWrapper):
    form = GroupAddForm



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.contentrules.browser.formhelper import EditForm
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.utils import getToolByName
from z3c.form import form
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class IRoleCondition(Interface):
    """Interface for the configurable aspects of a role condition.

    This is also used to create add and edit forms, below.
    """

    role_names = schema.Set(
        title=_(u'Roles'),
        description=_(u'The roles to check for.'),
        required=True,
        value_type=schema.Choice(vocabulary='plone.app.vocabularies.Roles')
    )


@implementer(IRoleCondition, IRuleElementData)
class RoleCondition(SimpleItem):
    """The actual persistent implementation of the role condition element.

    Note that we must mix in SimpleItem to keep Zope 2 security happy.
    """

    role_names = []
    element = 'plone.conditions.Role'

    @property
    def summary(self):
        return _(
            u'Roles are: ${names}',
            mapping=dict(names=', '.join(self.role_names))
        )


@implementer(IExecutable)
@adapter(Interface, IRoleCondition, Interface)
class RoleConditionExecutor(object):
    """The executor for this condition.

    This is registered as an adapter in configure.zcml
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        portal_membership = getToolByName(
            self.context, 'portal_membership', None)
        if portal_membership is None:
            return False
        member = portal_membership.getAuthenticatedMember()
        roles_in_context = member.getRolesInContext(
            aq_inner(self.event.object))
        for r in self.element.role_names:
            if r in roles_in_context:
                return True
        return False


class RoleAddForm(AddForm):
    """An add form for role rule conditions.
    """
    schema = IRoleCondition
    label = _(u'Add Role Condition')
    description = _(
        u'A role condition can prevent rules from executing unless '
        u'the current user has a particular role.'
    )
    form_name = _(u'Configure element')

    def create(self, data):
        c = RoleCondition()
        form.applyChanges(self, c, data)
        return c


class RoleAddFormView(ContentRuleFormWrapper):
    form = RoleAddForm


class RoleEditForm(EditForm):
    """An edit form for role conditions
    """
    schema = IRoleCondition
    label = _(u'Add Role Condition')
    description = _(
        u'A role condition can prevent rules from executing unless '
        u'the current user has a particular role.'
    )
    form_name = _(u'Configure element')


class RoleEditFormView(ContentRuleFormWrapper):
    form = RoleEditForm



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer

import plone.app.contentrules


class PloneAppContentrulesLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        self.loadZCML('testing.zcml', package=plone.app.contentrules.tests)


PLONE_APP_CONTENTRULES_FIXTURE = PloneAppContentrulesLayer()


PLONE_APP_CONTENTRULES_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENTRULES_FIXTURE,),
    name='PloneAppContentrulesLayer:IntegrationTesting',
)


PLONE_APP_CONTENTRULES_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENTRULES_FIXTURE,),
    name='PloneAppContentrulesLayer:FunctionalTesting',
)



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from email import message_from_string
from plone.app.contentrules.actions.mail import MailAction
from plone.app.contentrules.actions.mail import MailAddFormView
from plone.app.contentrules.actions.mail import MailEditFormView
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleAction
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from Products.CMFPlone.tests.utils import MockMailHost
from Products.MailHost.interfaces import IMailHost
from zope.component import getMultiAdapter
from zope.component import getSiteManager
from zope.component import getUtility
from zope.component.interfaces import IObjectEvent
from zope.interface import implementer

import unittest


@implementer(IObjectEvent)
class DummyEvent(object):

    def __init__(self, object):
        self.object = object


class TestMailAction(ContentRulesTestCase):

    def setUp(self):
        super(TestMailAction, self).setUp()
        self.folder['d1'].setTitle(u'Wälkommen')

        users = (
            ('userone', 'User One', 'user@one.com', ('Manager', 'Member')),
            ('usertwo', 'User Two', 'user@two.com', ('Reviewer', 'Member')),
            ('userthree', 'User Three', 'user@three.com', ('Owner', 'Member')),
            ('userfour', 'User Four', 'user@four.com', ('Member', )),
        )
        for id, fname, email, roles in users:
            self.portal.portal_membership.addMember(id, 'secret', roles, [])
            member = self.portal.portal_membership.getMemberById(id)
            member.setMemberProperties({'fullname': fname, 'email': email})
        # XXX: remove the manager role that was set in the base class
        setRoles(self.portal, TEST_USER_ID, [])

    def _setup_mockmail(self):
        sm = getSiteManager(self.portal)
        sm.unregisterUtility(provided=IMailHost)
        dummyMailHost = MockMailHost('MailHost')
        sm.registerUtility(dummyMailHost, IMailHost)
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = dummyMailHost
        return dummyMailHost

    def _teardown_mockmail(self):
        self.portal.MailHost = self.portal._original_MailHost
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(aq_base(self.portal._original_MailHost),
                           provided=IMailHost)

    def testRegistered(self):
        element = getUtility(IRuleAction, name='plone.actions.Mail')
        self.assertEqual('plone.actions.Mail', element.addview)
        self.assertEqual('edit', element.editview)
        self.assertEqual(None, element.for_)

    def testInvokeAddView(self):
        element = getUtility(IRuleAction, name='plone.actions.Mail')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter((rule, self.portal.REQUEST), name='+action')
        addview = getMultiAdapter((adding, self.portal.REQUEST),
                                  name=element.addview)
        self.assertTrue(isinstance(addview, MailAddFormView))

        addview.form_instance.update()
        output = addview.form_instance()
        self.assertIn('<h1>Substitutions</h1>', output)
        content = addview.form_instance.create(
            data={
                'subject': 'My Subject',
                'source': 'foo@bar.be',
                'recipients': 'foo@bar.be,bar@foo.be',
                'message': 'Hey, Oh!',
            }
        )
        addview.form_instance.add(content)

        e = rule.actions[0]
        self.assertTrue(isinstance(e, MailAction))
        self.assertEqual('My Subject', e.subject)
        self.assertEqual('foo@bar.be', e.source)
        self.assertEqual('foo@bar.be,bar@foo.be', e.recipients)
        self.assertEqual('Hey, Oh!', e.message)

    def testInvokeEditView(self):
        element = getUtility(IRuleAction, name='plone.actions.Mail')
        e = MailAction()
        editview = getMultiAdapter((e, self.folder.REQUEST),
                                   name=element.editview)
        self.assertTrue(isinstance(editview, MailEditFormView))

    def testExecute(self):
        # this avoids sending mail as currentuser@foobar.com
        self.portal.portal_membership.getAuthenticatedMember().setProperties(
            email='currentuser@foobar.com')
        dummyMailHost = self._setup_mockmail()
        e = MailAction()
        e.source = '$user_email'
        e.recipients = 'bar@foo.be, bar@foo.be, $reviewer_emails, ' \
                       '$manager_emails, $member_emails'
        e.message = u"Päge '${title}' created in ${url} !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)),
                             IExecutable)
        ex()
        sent_mails = {}
        for mail_sent in dummyMailHost.messages:
            mail_sent_msg = message_from_string(mail_sent)
            sent_mails[mail_sent_msg.get('To')] = mail_sent_msg

        mailSent = sent_mails['bar@foo.be']
        self.assertEqual('text/plain; charset="utf-8"',
                         mailSent.get('Content-Type'))
        self.assertEqual('currentuser@foobar.com', mailSent.get('From'))
        # The output message should be a utf-8 encoded string
        self.assertEqual(
            u"Päge 'Wälkommen' created in http://nohost/plone/f1/d1 !",
            mailSent.get_payload(decode=True).decode('utf8'))

        # check interpolation of $reviewer_emails
        self.assertTrue('user@two.com' in sent_mails)

        # check interpolation of $manager_emails
        self.assertTrue('user@one.com' in sent_mails)

        # check interpolation of $member_emails
        emails = [
            'bar@foo.be',
            'user@one.com',
            'user@two.com',
            'user@three.com',
            'user@four.com',
        ]
        self.assertEqual(
            set(emails),
            set(sent_mails.keys()))
        self._teardown_mockmail()

    def testExecuteNoSource(self):
        dummyMailHost = self._setup_mockmail()
        e = MailAction()
        e.recipients = 'bar@foo.be,foo@bar.be'
        e.message = 'Document created !'
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)),
                             IExecutable)
        # this no longer errors since it breaks usability
        self.assertTrue(ex)
        # and will return False for the unsent message
        self.assertEqual(ex(), False)
        # if we provide a site mail address the message sends correctly
        registry = getUtility(IRegistry)
        mail_settings = registry.forInterface(IMailSchema, prefix='plone')
        mail_settings.email_from_address = 'manager@portal.be'
        mail_settings.email_from_name = u'plone@rulez'
        ex()
        self.assertEqual(len(dummyMailHost.messages), 2)
        mailSent = message_from_string(dummyMailHost.messages[0])
        self.assertEqual('text/plain; charset="utf-8"',
                         mailSent.get('Content-Type'))
        self.assertIn(mailSent.get('To'), ['bar@foo.be', 'foo@bar.be'])
        self.assertEqual('"plone@rulez" <manager@portal.be>',
                         mailSent.get('From'))
        self.assertEqual('Document created !',
                         mailSent.get_payload())
        self._teardown_mockmail()

    def testExecuteMultiRecipients(self):
        dummyMailHost = self._setup_mockmail()
        e = MailAction()
        e.source = 'foo@bar.be'
        e.recipients = 'bar@foo.be,foo@bar.be'
        e.message = 'Document created !'
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)),
                             IExecutable)
        ex()
        self.assertEqual(len(dummyMailHost.messages), 2)
        # in py3 the order of mails is non-determininistic
        # because sending iterates over a set of recipients
        for msg in dummyMailHost.messages:
            if 'bar@foo.be' in msg:
                mailSent1 = message_from_string(msg)
            else:
                mailSent2 = message_from_string(msg)
        self.assertEqual('text/plain; charset="utf-8"',
                         mailSent1.get('Content-Type'))
        self.assertEqual('bar@foo.be', mailSent1.get('To'))
        self.assertEqual('foo@bar.be', mailSent1.get('From'))
        self.assertEqual('Document created !',
                         mailSent1.get_payload())
        self.assertEqual('text/plain; charset="utf-8"',
                         mailSent2.get('Content-Type'))
        self.assertEqual('foo@bar.be', mailSent2.get('To'))
        self.assertEqual('foo@bar.be', mailSent2.get('From'))
        self.assertEqual('Document created !',
                         mailSent2.get_payload())
        self._teardown_mockmail()

    def testExecuteExcludeActor(self):
        dummyMailHost = self._setup_mockmail()
        self.portal.portal_membership.getAuthenticatedMember().setProperties(
            email='currentuser@foobar.com')
        e = MailAction()
        e.source = '$user_email'
        e.exclude_actor = True
        e.recipients = 'bar@foo.be, currentuser@foobar.com'
        e.message = u'A dummy event just happened !!!!!'
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)),
                             IExecutable)
        ex()
        self.assertEqual(len(dummyMailHost.messages), 1)

        mailSent = message_from_string(dummyMailHost.messages[0])
        self.assertEqual('bar@foo.be', mailSent.get('To'))
        self._teardown_mockmail()

    def testExecuteNoRecipients(self):
        # no recipient
        dummyMailHost = self._setup_mockmail()
        e = MailAction()
        e.source = 'foo@bar.be'
        e.recipients = ''
        e.message = 'Document created !'
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)),
                             IExecutable)
        ex()
        self.assertEqual(len(dummyMailHost.messages), 0)
        self._teardown_mockmail()

    @unittest.skip(
        'Monkey patching does not work well with mocking. Needs fixing.'
    )
    def testExecuteBadMailHost(self):
        # Our goal is that mailing errors should not cause exceptions
        self.portal.portal_membership.getAuthenticatedMember().setProperties(
            email='currentuser@foobar.com')
        e = MailAction()
        e.source = '$user_email'
        e.recipients = 'bar@foo.be, $reviewer_emails, $manager_emails, ' \
                       '$member_emails'
        e.message = u"PÃ¤ge '${title}' created in ${url} !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)),
                             IExecutable)
        ex()



# -*- coding: utf-8 -*-
from plone.app.contentrules.actions.logger import LoggerAction
from plone.app.contentrules.actions.logger import LoggerEditFormView
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleAction
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.interfaces import IObjectEvent
from zope.interface import implementer
from zope.interface import Interface


@implementer(Interface)
class DummyEvent(object):
    pass


@implementer(IObjectEvent)
class DummyObjectEvent(object):

    def __init__(self, obj):
        self.object = obj


class TestLoggerAction(ContentRulesTestCase):

    def testRegistered(self):
        element = getUtility(IRuleAction, name='plone.actions.Logger')
        self.assertEqual('plone.actions.Logger', element.addview)
        self.assertEqual('edit', element.editview)
        self.assertEqual(None, element.for_)
        self.assertEqual(None, element.event)

    def testInvokeAddView(self):
        element = getUtility(IRuleAction, name='plone.actions.Logger')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter((rule, self.portal.REQUEST), name='+action')
        addview = getMultiAdapter(
            (adding, self.portal.REQUEST), name=element.addview)

        addview.form_instance.update()
        content = addview.form_instance.create(
            data={'targetLogger': 'foo', 'loggingLevel': 10, 'message': 'bar'})
        addview.form_instance.add(content)

        e = rule.actions[0]
        self.assertTrue(isinstance(e, LoggerAction))
        self.assertEqual('foo', e.targetLogger)
        self.assertEqual(10, e.loggingLevel)
        self.assertEqual('bar', e.message)

    def testInvokeEditView(self):
        element = getUtility(IRuleAction, name='plone.actions.Logger')
        e = LoggerAction()
        editview = getMultiAdapter(
            (e, self.folder.REQUEST), name=element.editview)
        self.assertTrue(isinstance(editview, LoggerEditFormView))

    def testProcessedMessage(self):
        e = LoggerAction()
        e.targetLogger = 'testing'
        e.loggingLevel = 0
        e.message = 'Test log event'
        ex = getMultiAdapter(
            (self.folder, e, DummyObjectEvent(self.folder)), IExecutable)
        self.assertEqual('Test log event', ex.processedMessage())

        e.message = 'Test log event : &c'
        self.assertEqual(
            'Test log event : <Folder at /plone/f1>',
            ex.processedMessage(),
        )

        e.message = 'Test log event : &e'
        self.assertEqual(
            'Test log event : '
            'plone.app.contentrules.tests.test_action_logger.DummyObjectEvent',
            ex.processedMessage(),
        )

        e.message = 'Test log event : &u'
        self.assertEqual(
            'Test log event : {0}'.format(TEST_USER_NAME),
            ex.processedMessage(),
        )

    def testExecute(self):
        e = LoggerAction()
        e.targetLogger = 'testing'
        e.loggingLevel = 0
        e.message = 'Test log event'
        ex = getMultiAdapter((self.folder, e, DummyEvent()), IExecutable)
        self.assertTrue(ex())



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules import api
from plone.contentrules.engine.interfaces import IRuleAssignmentManager
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.memoize.instance import memoize
from Products.CMFCore.interfaces import ISiteRoot
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory


class ManageAssignments(BrowserView):
    """Manage contextual rule assignments
    """

    template = ViewPageTemplateFile('templates/manage-assignments.pt')

    def __call__(self):
        context = aq_inner(self.context)
        request = aq_inner(self.request)
        form = request.form
        status = IStatusMessage(self.request)

        operation = request.get('operation', None)

        if operation == 'move_up':
            assignable = IRuleAssignmentManager(context)
            rule_id = request.get('rule_id')
            keys = list(assignable.keys())
            idx = keys.index(rule_id)
            del keys[idx]
            keys.insert(idx - 1, rule_id)
            assignable.updateOrder(keys)
        elif operation == 'move_down':
            assignable = IRuleAssignmentManager(context)
            rule_id = request.get('rule_id')
            keys = list(assignable.keys())
            idx = keys.index(rule_id)
            del keys[idx]
            keys.insert(idx + 1, rule_id)
            assignable.updateOrder(keys)
        elif 'form.button.AddAssignment' in form:
            rule_id = form.get('rule_id')
            api.assign_rule(self.context, rule_id)
        elif 'form.button.Delete' in form:
            rule_ids = form.get('rule_ids', ())
            for r in rule_ids:
                api.unassign_rule(self.context, r)

            status.addStatusMessage(_(u'Assignments deleted.'), type='info')
        elif 'form.button.Enable' in form:
            rule_ids = form.get('rule_ids', ())
            for r in rule_ids:
                api.edit_rule_assignment(context, r, enabled=True)

            status.addStatusMessage(_(u'Assignments enabled.'), type='info')
        elif 'form.button.Disable' in form:
            rule_ids = form.get('rule_ids', ())
            for r in rule_ids:
                api.edit_rule_assignment(context, r, enabled=False)

            status.addStatusMessage(_(u'Assignments disabled.'), type='info')
        elif 'form.button.Bubble' in form:
            rule_ids = form.get('rule_ids', ())
            for r in rule_ids:
                api.edit_rule_assignment(
                    context, r, bubbles=True, enabled=True)

            status.addStatusMessage(_(u'Changes saved.'), type='info')
        elif 'form.button.NoBubble' in form:
            rule_ids = form.get('rule_ids', ())
            for r in rule_ids:
                api.edit_rule_assignment(context, r, bubbles=False)

            status.addStatusMessage(_(u'Changes saved.'), type='info')

        return self.template()

    def globally_enabled(self):
        storage = getUtility(IRuleStorage)
        return storage.active

    @memoize
    def view_url(self):
        return self.context.absolute_url() + '/@@manage-content-rules'

    @memoize
    def type_name(self):
        context = aq_inner(self.context)
        fti = context.getTypeInfo()
        return fti.Title()

    @memoize
    def acquired_rules(self):

        # Short circuit if this is the root of the portal
        if ISiteRoot.providedBy(self.context):
            return []

        in_use = set([r['id'] for r in self.assigned_rules()])

        storage = getUtility(IRuleStorage)
        events = self._events()

        assignments = []
        context = aq_parent(aq_inner(self.context))

        while context is not None:
            assignable = IRuleAssignmentManager(context, None)
            if assignable is not None:
                for key, assignment in assignable.items():
                    if key not in in_use and assignment.bubbles:
                        rule = storage.get(key, None)
                        if rule is not None:
                            url = '{0}/@@manage-content-rules'.format(
                                context.absolute_url(),
                            )
                            assignments.append(dict(
                                id=key,
                                title=rule.title,
                                description=rule.description,
                                trigger=events.get(rule.event, 'Unknown'),
                                url=url,
                                enabled=(assignment.enabled and rule.enabled),
                            ))
            if ISiteRoot.providedBy(context):
                context = None
            else:
                context = aq_parent(context)

        return assignments

    @memoize
    def assigned_rules(self):
        assignable = IRuleAssignmentManager(self.context)
        storage = getUtility(IRuleStorage)
        events = self._events()

        assignments = []
        for key, assignment in assignable.items():
            rule = storage.get(key, None)
            if rule is not None:
                assignments.append(dict(id=key,
                                        title=rule.title,
                                        description=rule.description,
                                        trigger=events.get(
                                            rule.event,
                                            'Unknown'
                                        ),
                                        url=self._rule_url(key),
                                        bubbles=assignment.bubbles,
                                        enabled=assignment.enabled,
                                        global_enabled=rule.enabled, ))
        return assignments

    def has_rules(self):
        return len(self.assigned_rules()) > 0 or len(self.acquired_rules()) > 0

    def assignable_rules(self):
        in_use = set([r['id'] for r in self.assigned_rules()])
        assignable = []
        for key, rule in getUtility(IRuleStorage).items():
            if key not in in_use:
                assignable.append(dict(id=key,
                                       title=rule.title,
                                       description=rule.description, ))
        return assignable

    @memoize
    def _events(self):
        eventsFactory = getUtility(
            IVocabularyFactory, name='plone.contentrules.events')
        return dict([(e.value, e.token) for e in eventsFactory(self.context)])

    def _rule_url(self, key):
        return '{0}/++rule++{1}/@@manage-elements'.format(
            self._portal_url(),
            key,
        )

    @memoize
    def _portal_url(self):
        portal_state = getMultiAdapter(
            (self.context, self.request), name='plone_portal_state')
        return portal_state.portal_url()



# -*- coding: utf-8 -*-
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IRule
from Products.CMFCore.interfaces import ISiteRoot
from zope.component import adapter
from zope.component import getUtility
from zope.interface import implementer
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.traversing.interfaces import ITraversable


@adapter(ISiteRoot, IBrowserRequest)
@implementer(ITraversable)
class RuleNamespace(object):
    """Used to traverse to a rule.

    Traversing to portal/++rule++foo will retrieve the rule with id 'foo'
    stored in context, acquisition-wrapped.
    """

    def __init__(self, context, request=None):
        self.context = context
        self.request = request

    def traverse(self, name, ignore):
        manager = getUtility(IRuleStorage)
        return manager[name]


@adapter(IRule, IBrowserRequest)
@implementer(ITraversable)
class RuleConditionNamespace(object):
    """Used to traverse to a rule condition

    Traversing to portal/++rule++foo/++condition++1 will retrieve the second
    condition of the rule rule with id 'foo', acquisition-wrapped.
    """

    def __init__(self, context, request=None):
        self.context = context
        self.request = request

    def traverse(self, name, ignore):
        condition = self.context.conditions[int(name)]
        traversal_id = '++condition++{0}'.format(name)
        if condition.id != traversal_id:
            condition.__name__ = condition.id = traversal_id
        return condition


@adapter(IRule, IBrowserRequest)
@implementer(ITraversable)
class RuleActionNamespace(object):
    """Used to traverse to a rule condition

    Traversing to portal/++rule++foo/++action++1 will retrieve the second
    condition of the rule rule with id 'foo', acquisition-wrapped.
    """

    def __init__(self, context, request=None):
        self.context = context
        self.request = request

    def traverse(self, name, ignore):
        action = self.context.actions[int(name)]
        traversal_id = '++action++{0}'.format(name)
        if action.id != traversal_id:
            action.__name__ = action.id = traversal_id
        return action



# -*- coding: utf-8 -*-
from zope.container.contained import NameChooser
from zope.container.interfaces import INameChooser
from zope.interface import implementer


ATTEMPTS = 100


@implementer(INameChooser)
class RuleNameChooser(NameChooser):
    """A name chooser for content rules.
    """

    def __init__(self, context):
        self.context = context

    def chooseName(self, name, object):
        container = self.context

        if not name:
            name = object.__class__.__name__.lower()

        i = 1
        new_name = '{0}-{1}'.format(name, i)
        while new_name in container and i <= ATTEMPTS:
            i += 1
            new_name = '{0}-{1}'.format(name, i)

        self.checkName(new_name, object)
        return new_name



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.actions import ActionAddForm
from plone.app.contentrules.actions import ActionEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.utils import getToolByName
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class IVersioningAction(Interface):
    """Interface for the configurable aspects of a versioning action.

    This is also used to create add and edit forms, below.
    """

    comment = schema.TextLine(
        title=_(u'Comment'),
        description=_(
            u'The comment added to the history while versioning the content.'),
        required=False,
    )


@implementer(IVersioningAction, IRuleElementData)
class VersioningAction(SimpleItem):
    """The actual persistent implementation of the versioning action element.
    """

    comment = ''

    element = 'plone.actions.Versioning'

    @property
    def summary(self):
        return _(
            u'Versioning with comment ${comment}',
            mapping=dict(comment=self.comment),
        )


@adapter(Interface, IVersioningAction, Interface)
@implementer(IExecutable)
class VersioningActionExecutor(object):
    """The executor for this action.

    This is registered as an adapter in configure.zcml
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        comment = _(self.element.comment)
        pr = getToolByName(self.context, 'portal_repository')
        pr.save(obj=self.event.object, comment=comment)
        return True


class VersioningAddForm(ActionAddForm):
    """An add form for versioning rule actions.
    """
    schema = IVersioningAction
    label = _(u'Add Versioning Action')
    description = _(u'A versioning action will store a version of a content '
                    u'no matter versioning is enabled for it or not.')
    form_name = _(u'Configure element')
    Type = VersioningAction


class VersioningAddFormView(ContentRuleFormWrapper):
    form = VersioningAddForm


class VersioningEditForm(ActionEditForm):
    """An edit form for versioning rule actions.

    z3c.form does all the magic here.
    """
    schema = IVersioningAction
    label = _(u'Edit Versioning Action')
    description = _(u'A versioning action will store a version of a content '
                    u'no matter versioning is enabled for it or not.')
    form_name = _(u'Configure element')


class VersioningEditFormView(ContentRuleFormWrapper):
    form = VersioningEditForm



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.actions import ActionAddForm
from plone.app.contentrules.actions import ActionEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import utils
from Products.statusmessages.interfaces import IStatusMessage
from ZODB.POSException import ConflictError
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class IWorkflowAction(Interface):
    """Interface for the configurable aspects of a workflow action.

    This is also used to create add and edit forms, below.
    """

    transition = schema.Choice(
        title=_(u'Transition'),
        description=_(u'Select the workflow transition to attempt'),
        required=True,
        vocabulary='plone.app.vocabularies.WorkflowTransitions',
    )


@implementer(IWorkflowAction, IRuleElementData)
class WorkflowAction(SimpleItem):
    """The actual persistent implementation of the action element.
    """

    transition = ''
    element = 'plone.actions.Workflow'

    @property
    def summary(self):
        return _(
            u'Execute transition ${transition}',
            mapping=dict(transition=self.transition),
        )


@adapter(Interface, IWorkflowAction, Interface)
@implementer(IExecutable)
class WorkflowActionExecutor(object):
    """The executor for this action.
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        portal_workflow = getToolByName(self.context, 'portal_workflow', None)
        if portal_workflow is None:
            return False

        obj = self.event.object

        try:
            portal_workflow.doActionFor(obj, self.element.transition)
        except ConflictError as e:
            raise e
        except Exception as e:
            self.error(obj, str(e))
            return False

        return True

    def error(self, obj, error):
        request = getattr(self.context, 'REQUEST', None)
        if request is not None:
            title = utils.safe_unicode(utils.pretty_title_or_id(obj, obj))
            error = utils.safe_unicode(error)
            message = _(
                u"Unable to change state of ${name} as part of content rule 'workflow' action: ${error}",  # noqa
                mapping={'name': title, 'error': error})
            IStatusMessage(request).addStatusMessage(message, type='error')


class WorkflowAddForm(ActionAddForm):
    """An add form for workflow actions.
    """
    schema = IWorkflowAction
    label = _(u'Add Workflow Action')
    description = _(
        u'A workflow action triggers a workflow transition on an object.')
    form_name = _(u'Configure element')
    Type = WorkflowAction


class WorkflowAddFormView(ContentRuleFormWrapper):
    form = WorkflowAddForm


class WorkflowEditForm(ActionEditForm):
    """An edit form for workflow rule actions.
    """
    schema = IWorkflowAction
    label = _(u'Edit Workflow Action')
    description = _(
        u'A workflow action triggers a workflow transition on an object.')
    form_name = _(u'Configure element')


class WorkflowEditFormView(ContentRuleFormWrapper):
    form = WorkflowAddForm



# -*- coding: utf-8 -*-
from plone.app.contentrules.testing import PLONE_APP_CONTENTRULES_FUNCTIONAL_TESTING
from plone.testing import layered

import doctest
import unittest


optionflags = (doctest.NORMALIZE_WHITESPACE |
               doctest.ELLIPSIS |
               doctest.REPORT_NDIFF)


def test_suite():
    suite = unittest.TestSuite()
    for doc in ['assignment.txt', 'simplepublish.txt', 'multipublish.txt']:
        suite.addTest(layered(
            doctest.DocFileSuite(doc, package='plone.app.contentrules.tests',
                                 optionflags=optionflags),
            layer=PLONE_APP_CONTENTRULES_FUNCTIONAL_TESTING))
    return suite



# -*- coding: utf-8 -*-
from plone.app.contentrules import handlers
from plone.app.contentrules.tests import dummy
from plone.app.contentrules.tests.base import ContentRulesTestCase
from zope.event import notify
from zope.lifecycleevent import ObjectCreatedEvent


class TestDuplicateRuleFilter(ContentRulesTestCase):

    def setUp(self):
        super(TestDuplicateRuleFilter, self).setUp()
        self.context = self.folder
        self.event = dummy.DummyEvent(self.context)
        self.rulefilter = handlers.DuplicateRuleFilter()
        self.rule = dummy.DummyRule()
        self.uuidaware = dummy.DummyNonArchetypesContext()
        notify(ObjectCreatedEvent(self.uuidaware))

    def test_call_archetypescontext(self):
        to_execute = self.rulefilter(self.context, self.rule, self.event)
        self.assertTrue(to_execute)
        to_execute = self.rulefilter(self.context, self.rule, self.event)
        self.assertTrue(not to_execute)

    def test_call_two_events_in_same_context(self):
        # if events on two different objects are handled in the same context,
        # they are not filtered
        to_execute = self.rulefilter(self.context, self.rule, self.event)
        self.assertTrue(to_execute)
        self.portal.invokeFactory('Folder', 'folder2')
        event2 = dummy.DummyEvent(self.portal.folder2)
        to_execute = self.rulefilter(self.context, self.rule, event2)
        self.assertTrue(to_execute)

    def test_call_uuidaware(self):
        to_execute = self.rulefilter(self.uuidaware, self.rule, self.event)
        self.assertTrue(to_execute)
        to_execute = self.rulefilter(self.uuidaware, self.rule, self.event)
        self.assertTrue(not to_execute)

    def test_delayed_events(self):
        # many events can be delayed
        # But this was only true for Archetypes content.
        # The tests no use dexterity so we skip it:
        return

        self.portal.invokeFactory('Folder', 'folder2')
        event1 = dummy.DummyEvent(self.folder)
        event2 = dummy.DummyEvent(self.portal.folder2)
        from plone.app.contentrules.handlers import _status
        _status.delayed_events = {}
        handlers.added(event1)
        handlers.added(event2)
        from plone.app.contentrules.handlers import _status
        self.assertEqual(len(_status.delayed_events), 2)



# -*- coding: utf-8 -*-
from plone.app.contentrules.actions.versioning import VersioningAction
from plone.app.contentrules.actions.versioning import VersioningEditFormView
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleAction
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import implementer
from zope.interface import Interface


@implementer(Interface)
class DummyEvent(object):

    def __init__(self, object):
        self.object = object


class TestVersioningAction(ContentRulesTestCase):

    def testRegistered(self):
        element = getUtility(IRuleAction, name='plone.actions.Versioning')
        self.assertEqual('plone.actions.Versioning', element.addview)
        self.assertEqual('edit', element.editview)
        self.assertEqual(None, element.for_)
        self.assertEqual(None, element.event)

    def testInvokeAddView(self):
        element = getUtility(IRuleAction, name='plone.actions.Versioning')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter((rule, self.request), name='+action')
        addview = getMultiAdapter((adding, self.request), name=element.addview)

        addview.form_instance.update()
        content = addview.form_instance.create(data={'comment': 'Hello world'})
        addview.form_instance.add(content)

        e = rule.actions[0]
        self.assertTrue(isinstance(e, VersioningAction))
        self.assertEqual('Hello world', e.comment)

    def testInvokeEditView(self):
        element = getUtility(IRuleAction, name='plone.actions.Versioning')
        e = VersioningAction()
        editview = getMultiAdapter((e, self.request), name=element.editview)
        self.assertTrue(isinstance(editview, VersioningEditFormView))

    def testExecute(self):
        e = VersioningAction()
        e.comment = 'Hello world'

        ex = getMultiAdapter(
            (self.folder, e, DummyEvent(self.folder)), IExecutable)
        # not version for now
        pr = self.portal.portal_repository
        self.assertEqual(pr.getHistoryMetadata(self.folder), [])

        # action will create first version
        self.assertEqual(True, ex())
        self.assertEqual(
            pr.getHistoryMetadata(self.folder).getLength(countPurged=False), 1)
        # calling action again will create a second version
        ex()
        self.assertEqual(
            pr.getHistoryMetadata(self.folder).getLength(countPurged=False), 2)



# -*- coding: utf-8 -*-
from plone.app.contentrules.conditions.portaltype import PortalTypeCondition
from plone.app.contentrules.conditions.portaltype import PortalTypeEditFormView
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleCondition
from Products.CMFCore.interfaces import ITypesTool
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.interfaces import IObjectEvent
from zope.interface import implementer


@implementer(IObjectEvent)
class DummyEvent(object):

    def __init__(self, obj):
        self.object = obj


class TestPortalTypeCondition(ContentRulesTestCase):

    def testRegistered(self):
        element = getUtility(
            IRuleCondition, name='plone.conditions.PortalType')
        self.assertEqual('plone.conditions.PortalType', element.addview)
        self.assertEqual('edit', element.editview)
        self.assertEqual(None, element.for_)
        self.assertEqual(IObjectEvent, element.event)

    def testInvokeAddView(self):
        element = getUtility(
            IRuleCondition, name='plone.conditions.PortalType')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter(
            (rule, self.portal.REQUEST), name='+condition')
        addview = getMultiAdapter(
            (adding, self.portal.REQUEST), name=element.addview)

        addview.form_instance.update()
        content = addview.form_instance.create(
            data={'check_types': ['Folder', 'Image']})
        addview.form_instance.add(content)

        e = rule.conditions[0]
        self.assertTrue(isinstance(e, PortalTypeCondition))
        self.assertEqual(['Folder', 'Image'], e.check_types)

    def testInvokeEditView(self):
        element = getUtility(
            IRuleCondition, name='plone.conditions.PortalType')
        e = PortalTypeCondition()
        editview = getMultiAdapter(
            (e, self.folder.REQUEST), name=element.editview)
        self.assertTrue(isinstance(editview, PortalTypeEditFormView))

    def testExecute(self):
        e = PortalTypeCondition()
        e.check_types = ['Folder', 'Image']

        ex = getMultiAdapter(
            (self.portal, e, DummyEvent(self.folder)), IExecutable)
        self.assertTrue(ex())

        ex = getMultiAdapter(
            (self.portal, e, DummyEvent(self.portal)), IExecutable)
        self.assertFalse(ex())

        self.folder.portal_types = None
        self.portal.getSiteManager().unregisterUtility(provided=ITypesTool)
        ex = getMultiAdapter(
            (self.portal, e, DummyEvent(self.folder)), IExecutable)
        self.assertFalse(ex())



# -*- coding: utf-8 -*-
"""Base class for integration tests, based on plone.app.testing
"""
from plone.app.contentrules.testing import PLONE_APP_CONTENTRULES_INTEGRATION_TESTING  # noqa: E501
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from zope.component import getMultiAdapter

import unittest


class ContentRulesTestCase(unittest.TestCase):
    """Base class for integration tests for plone.app.contentrules.
    This may provide specific set-up and tear-down operations, or provide
    convenience methods.
    """

    layer = PLONE_APP_CONTENTRULES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'f1')
        self.folder = self.portal['f1']
        self.folder.invokeFactory('Document', 'd1')
        self.portal.invokeFactory('Folder', 'target')

    def addAuthToRequest(self):
        portal = self.layer['portal']
        request = self.layer['request']
        authenticator = getMultiAdapter(
            (portal, request), name=u'authenticator')
        auth = authenticator.authenticator().split('value="')[1].rstrip('"/>')
        request.form['_authenticator'] = auth



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.interfaces import IContentRulesForm
from plone.autoform.form import AutoExtensibleForm
from plone.z3cform import layout
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form import form
from zope.component import getMultiAdapter
from zope.event import notify
from zope.interface import implementer

import zope.lifecycleevent


@implementer(IContentRulesForm)
class AddForm(AutoExtensibleForm, form.AddForm):
    """A base add form for content rule.

    Use this for rule elements that require configuration before being added to
    a rule. Element types that do not should use NullAddForm instead.

    Sub-classes should define create() and set the form_fields class variable.

    Notice the suble difference between AddForm and NullAddform in that the
    create template method for AddForm takes as a parameter a dict 'data':

        def create(self, data):
            return MyAssignment(data.get('foo'))

    whereas the NullAddForm has no data parameter:

        def create(self):
            return MyAssignment()
    """
    ignoreContext = True

    def updateActions(self):
        super(AddForm, self).updateActions()
        self.actions['save'].addClass('context')
        self.actions['cancel'].addClass('standalone')

    def nextURL(self):
        rule = aq_parent(aq_inner(self.context))
        context = aq_parent(aq_inner(rule))
        url = str(getMultiAdapter(
            (context, self.request), name=u'absolute_url'))
        focus = self.context.id.strip('+')
        return '{0}/++rule++{1}/@@manage-elements#{2}'.format(
            url,
            rule.__name__,
            focus,
        )

    def add(self, content):
        self.context.add(content)

    @button.buttonAndHandler(_(u'label_save', default=u'Save'), name='save')
    def handle_save_action(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        content = self.create(data)
        self.add(content)
        nextURL = self.nextURL()
        if nextURL:
            self.request.response.redirect(self.nextURL())

    @button.buttonAndHandler(
        _(u'label_cancel', default=u'Cancel'),
        name='cancel',
    )
    def handle_cancel_action(self, action):
        nextURL = self.nextURL()
        if nextURL:
            self.request.response.redirect(self.nextURL())
        return ''


class NullAddForm(BrowserView):
    """An add view that will add its content immediately, without presenting
    a form.

    You should subclass this for rule elements that do not require any
    configuration before being added, and write a create() method that takes no
    parameters and returns the appropriate assignment instance.
    """

    def __call__(self):
        ob = self.create()
        notify(zope.lifecycleevent.ObjectCreatedEvent(ob))
        self.context.add(ob)
        nextURL = self.nextURL()
        if nextURL:
            self.request.response.redirect(self.nextURL())
        return ''

    def nextURL(self):
        rule = aq_parent(aq_inner(self.context))
        context = aq_parent(aq_inner(rule))
        url = str(getMultiAdapter(
            (context, self.request), name=u'absolute_url'))
        return '{0}/++rule++{1}/@@manage-elements'.format(url, rule.__name__)

    def create(self):
        raise NotImplementedError('concrete classes must implement create()')


@implementer(IContentRulesForm)
class EditForm(AutoExtensibleForm, form.EditForm):
    """An edit form for rule elements.
    """

    def updateActions(self):
        super(EditForm, self).updateActions()
        self.actions['save'].addClass('context')
        self.actions['cancel'].addClass('standalone')

    @button.buttonAndHandler(_(u'label_save', default=u'Save'), name='save')
    def handle_save_action(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        self.applyChanges(data)
        nextURL = self.nextURL()
        if nextURL:
            self.request.response.redirect(self.nextURL())
        return ''

    @button.buttonAndHandler(
        _(u'label_cancel', default=u'Cancel'),
        name='cancel',
    )
    def handle_cancel_action(self, action):
        nextURL = self.nextURL()
        if nextURL:
            self.request.response.redirect(self.nextURL())
        return ''

    def nextURL(self):
        element = aq_inner(self.context)
        rule = aq_parent(element)
        context = aq_parent(rule)
        url = str(getMultiAdapter(
            (context, self.request), name=u'absolute_url'))
        focus = self.context.id.strip('+')
        return '{0}/++rule++{1}/@@manage-elements#{2}'.format(
            url,
            rule.__name__,
            focus,
        )


class ContentRuleFormWrapper(layout.FormWrapper):

    index = ViewPageTemplateFile('templates/contentrules-pageform.pt')



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.contentrules.browser.formhelper import EditForm
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from z3c.form import form
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


try:
    from Products.ATContentTypes.interfaces import IFileContent
except ImportError:
    IFileContent = None


class IFileExtensionCondition(Interface):
    """Interface for the configurable aspects of a portal type condition.

    This is also used to create add and edit forms, below.
    """

    file_extension = schema.TextLine(
        title=_(u'File extension'),
        description=_(u'The file extension to check for'),
        required=True
    )


@implementer(IFileExtensionCondition, IRuleElementData)
class FileExtensionCondition(SimpleItem):
    """The actual persistent implementation of the file extension condition.

    Note that we must mix in Explicit to keep Zope 2 security happy.
    """

    file_extension = u''
    element = 'plone.conditions.FileExtension'

    @property
    def summary(self):
        return _(
            u'File extension is ${ext}',
            mapping=dict(ext=self.file_extension)
        )


@implementer(IExecutable)
@adapter(Interface, IFileExtensionCondition, Interface)
class FileExtensionConditionExecutor(object):
    """The executor for this condition.

    This is registered as an adapter in configure.zcml
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        obj = self.event.object
        if IFileContent is None:
            return False
        if not IFileContent.providedBy(obj):
            return False

        base_unit = obj.getFile()
        get_filename = getattr(base_unit, 'getFilename', None)
        if not get_filename:
            return False

        name = get_filename()
        extension = name[name.rfind('.') + 1:]
        return extension == self.element.file_extension


class FileExtensionAddForm(AddForm):
    """An add form for file extension rule conditions.
    """
    schema = IFileExtensionCondition
    label = _(u'Add File Extension Condition')
    description = _(
        u'A file extension condition can restrict a rule from '
        u'executing unless the target is a File with a particular extension.'
    )
    form_name = _(u'Configure element')

    def create(self, data):
        c = FileExtensionCondition()
        form.applyChanges(self, c, data)
        return c


class FileExtensionAddFormView(ContentRuleFormWrapper):
    form = FileExtensionAddForm


class FileExtensionEditForm(EditForm):
    """An edit form for portal type conditions

    z3c.form does all the magic here.
    """
    schema = IFileExtensionCondition
    label = _(u'Edit File Extension Condition')
    description = _(
        u'A file extension condition can restrict a rule from '
        u'executing unless the target is a File with a particular extension.'
    )
    form_name = _(u'Configure element')


class FileExtensionEditFormView(ContentRuleFormWrapper):
    form = FileExtensionEditForm



# -*- coding: utf-8 -*-
from plone.app.contentrules.api import edit_rule_assignment
from plone.app.contentrules.testing import PLONE_APP_CONTENTRULES_FUNCTIONAL_TESTING
from plone.app.testing import applyProfile
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.dexterity.utils import createContentInContainer

import unittest


class TestCascadingRule(unittest.TestCase):

    layer = PLONE_APP_CONTENTRULES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'news')
        self.portal.invokeFactory('Folder', 'events')

        applyProfile(self.portal, 'plone.app.contentrules:testing')
        edit_rule_assignment(self.portal, 'test4', bubbles=1, enabled=1)
        edit_rule_assignment(self.portal, 'test5', bubbles=1, enabled=1)

    def test_cascading_rule(self):
        # check that test2 rule and test4 rule are executed
        # test2 rule publishes the event in news folder
        # test4 rule moves it in events folder when it is published

        createContentInContainer(self.portal.news, 'Event', id='my-event')
        self.assertFalse('my-event' in self.portal.news)
        self.assertTrue('my-event' in self.portal.events)

        wtool = self.portal.portal_workflow
        self.assertEqual(
            wtool.getInfoFor(self.portal.events['my-event'], 'review_state'),
            'published',
        )



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.actions import ActionAddForm
from plone.app.contentrules.actions import ActionEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.utils import getToolByName
from zope import schema
from zope.component import adapter
from zope.component.interfaces import IObjectEvent
from zope.interface import implementer
from zope.interface import Interface

import logging


logger = logging.getLogger('plone.contentrules.logger')
handler = logging.StreamHandler()
formatter = logging.Formatter(
    '%(asctime)s - %(name)s - %(levelname)s -  %(message)s'
)
handler.setFormatter(formatter)
logger.addHandler(handler)


class ILoggerAction(Interface):
    """Interface for the configurable aspects of a logger action.

    This is also used to create add and edit forms, below.
    """

    targetLogger = schema.ASCIILine(title=_(u'Logger name'),
                                    default='Plone')

    loggingLevel = schema.Int(title=_(u'Logging level'),
                              default=20)  # INFO

    message = schema.TextLine(
        title=_(u'Message'),
        description=_(
            'help_contentrules_logger_message',
            default=u'&e = the triggering event, '
                    u'&c = the context, &u = the user',
        ),
        default=_(
            'text_contentrules_logger_message',
            default=u'Caught &e at &c by &u',
        )
    )


@implementer(ILoggerAction, IRuleElementData)
class LoggerAction(SimpleItem):
    """The actual persistent implementation of the logger action element.

    Note that we must mix in Explicit to keep Zope 2 security happy.
    """

    targetLogger = ''
    loggingLevel = ''
    message = ''

    element = 'plone.actions.Logger'

    @property
    def summary(self):
        return _(u'Log message ${message}', mapping=dict(message=self.message))


@adapter(Interface, ILoggerAction, Interface)
@implementer(IExecutable)
class LoggerActionExecutor(object):
    """The executor for this action.

    This is registered as an adapter in configure.zcml
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def processedMessage(self):
        processedMessage = self.element.message
        if '&e' in processedMessage:
            event_class = self.event.__class__
            processedMessage = processedMessage.replace(
                '&e',
                '{0}.{1}'.format(
                    event_class.__module__,
                    event_class.__name__,
                )
            )

        if '&c' in processedMessage and IObjectEvent.providedBy(self.event):
            processedMessage = processedMessage.replace(
                '&c', repr(self.event.object))

        if '&u' in processedMessage:
            mtool = getToolByName(self.context, 'portal_membership')
            member = mtool.getAuthenticatedMember().getUserName()
            processedMessage = processedMessage.replace('&u', member)

        return processedMessage

    def __call__(self):
        logger = logging.getLogger(self.element.targetLogger)
        logger.log(self.element.loggingLevel, self.processedMessage())
        return True


class LoggerAddForm(ActionAddForm):
    """An add form for logger rule actions.
    """
    schema = ILoggerAction
    label = _(u'Add Logger Action')
    description = _(u'A logger action can output a message to the system log.')
    form_name = _(u'Configure element')
    Type = LoggerAction


class LoggerAddFormView(ContentRuleFormWrapper):
    form = LoggerAddForm


class LoggerEditForm(ActionEditForm):
    """An edit form for logger rule actions.

    z3c.form does all the magic here.
    """
    schema = ILoggerAction
    label = _(u'Edit Logger Action')
    description = _(u'A logger action can output a message to the system log.')
    form_name = _(u'Configure element')


class LoggerEditFormView(ContentRuleFormWrapper):
    form = LoggerAddForm



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from plone.contentrules.rule.interfaces import IRuleElementData
from plone.uuid.interfaces import IAttributeUUID
from zope.component.interfaces import IObjectEvent
from zope.interface import implementer


@implementer(IRuleElementData)
class DummyCondition(SimpleItem):
    element = 'dummy.condition'
    summary = 'Dummy condition'


@implementer(IRuleElementData)
class DummyAction(SimpleItem):
    element = 'dummy.action'
    summary = 'Dummy action'


@implementer(IObjectEvent)
class DummyEvent(object):

    def __init__(self, object):
        self.object = object


class DummyRule(object):

    def __init__(self, name='dummy'):
        self.__name__ = name


@implementer(IAttributeUUID)
class DummyNonArchetypesContext(object):
    pass



# -*- coding: utf-8 -*-
from plone.app.contentrules.tests.base import ContentRulesTestCase


class TestEvents(ContentRulesTestCase):

    def testEventHandlerExecutesRules(self):
        # XXX Test missing
        pass

    def testEventHandlerExecutesRulesOnlyOnce(self):
        # XXX Test missing
        pass



# -*- coding: utf-8 -*-
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.contentrules.engine.interfaces import IRuleAssignable
from plone.contentrules.rule.interfaces import IRuleEventType
from zope.lifecycleevent.interfaces import IObjectAddedEvent
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectRemovedEvent


class TestProductInstall(ContentRulesTestCase):

    def testRuleContainerInterfaces(self):
        self.assertTrue(IRuleAssignable.providedBy(self.folder))
        self.assertTrue(IRuleAssignable.providedBy(self.portal))

    def testEventTypesMarked(self):
        self.assertTrue(IRuleEventType.providedBy(IObjectAddedEvent))
        self.assertTrue(IRuleEventType.providedBy(IObjectModifiedEvent))
        self.assertTrue(IRuleEventType.providedBy(IObjectRemovedEvent))



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.contentrules.browser.formhelper import EditForm
from plone.app.contentrules.rule import Rule
from plone.contentrules.rule.interfaces import IRuleConfiguration
from Products.CMFPlone.utils import base_hasattr
from z3c.form.form import applyChanges
from zope.component import getMultiAdapter


class RuleAddForm(AddForm):
    """An add form for rules.
    """
    schema = IRuleConfiguration
    ignoreContext = True
    label = _(u'Add Rule')
    description = _(u'Add a new rule. Once complete, you can manage the '
                    u"rule's actions and conditions separately.")

    def nextURL(self):
        context = aq_parent(aq_inner(self.context))
        url = str(getMultiAdapter(
            (context, self.request), name=u'absolute_url'))
        if base_hasattr(self.context, '_chosen_name'):
            return '{0}/++rule++{1}/@@manage-elements'.format(
                url,
                self.context._chosen_name,
            )
        else:
            return '{0}/@@rules-controlpanel'.format(url)

    def create(self, data):
        rule = Rule()
        applyChanges(self, rule, data)
        return rule


class RuleAddFormView(ContentRuleFormWrapper):
    form = RuleAddForm


class RuleEditForm(EditForm):
    """An edit form for rules.
    """
    schema = IRuleConfiguration
    label = _(u'Edit Rule')
    description = _(u'Edit an existing rule.')

    def nextURL(self):
        context = aq_parent(aq_inner(self.context))
        url = str(getMultiAdapter(
            (context, self.request), name=u'absolute_url'))
        return url + '/@@rules-controlpanel'


class RuleEditFormView(ContentRuleFormWrapper):
    form = RuleEditForm



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from OFS.event import ObjectClonedEvent
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.actions import ActionAddForm
from plone.app.contentrules.actions import ActionEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.vocabularies.catalog import CatalogSource
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import utils
from Products.statusmessages.interfaces import IStatusMessage
from ZODB.POSException import ConflictError
from zope import schema
from zope.component import adapter
from zope.event import notify
from zope.interface import implementer
from zope.interface import Interface
from zope.lifecycleevent import ObjectCopiedEvent

import OFS.subscribers


class ICopyAction(Interface):
    """Interface for the configurable aspects of a move action.

    This is also used to create add and edit forms, below.
    """

    target_folder = schema.Choice(
        title=_(u'Target folder'),
        description=_(u'As a path relative to the portal root.'),
        required=True,
        source=CatalogSource(is_folderish=True),
    )


@implementer(ICopyAction, IRuleElementData)
class CopyAction(SimpleItem):
    """The actual persistent implementation of the action element.
    """

    target_folder = ''
    element = 'plone.actions.Copy'

    @property
    def summary(self):
        return _(u'Copy to folder ${folder}.',
                 mapping=dict(folder=self.target_folder))


@adapter(Interface, ICopyAction, Interface)
@implementer(IExecutable)
class CopyActionExecutor(object):
    """The executor for this action.
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        portal_url = getToolByName(self.context, 'portal_url', None)
        if portal_url is None:
            return False

        obj = self.event.object

        path = self.element.target_folder
        if len(path) > 1 and path[0] == '/':
            path = path[1:]
        target = portal_url.getPortalObject().unrestrictedTraverse(
            str(path),
            None,
        )

        if target is None:
            self.error(
                obj,
                _(
                    u'Target folder ${target} does not exist.',
                    mapping={'target': path}
                )
            )
            return False

        try:
            obj._notifyOfCopyTo(target, op=0)
        except ConflictError:
            raise
        except Exception as e:
            self.error(obj, str(e))
            return False

        old_id = obj.getId()
        new_id = self.generate_id(target, old_id)

        orig_obj = obj
        obj = obj._getCopy(target)
        obj._setId(new_id)

        notify(ObjectCopiedEvent(obj, orig_obj))

        target._setObject(new_id, obj)
        obj = target._getOb(new_id)
        obj.wl_clearLocks()

        obj._postCopy(target, op=0)

        OFS.subscribers.compatibilityCall('manage_afterClone', obj, obj)

        notify(ObjectClonedEvent(obj))

        return True

    def error(self, obj, error):
        request = getattr(self.context, 'REQUEST', None)
        if request is not None:
            title = utils.pretty_title_or_id(obj, obj)
            message = _(
                u'Unable to copy ${name} as part of content rule '
                u"'copy' action: ${error}",
                mapping={'name': title, 'error': error}
            )
            IStatusMessage(request).addStatusMessage(message, type='error')

    def generate_id(self, target, old_id):
        taken = getattr(aq_base(target), 'has_key', None)
        if taken is None:
            item_ids = set(target.objectIds())

            def taken(x): return x in item_ids
        if not taken(old_id):
            return old_id
        idx = 1
        while taken('{0}.{1}'.format(old_id, idx)):
            idx += 1
        return '{0}.{1}'.format(old_id, idx)


class CopyAddForm(ActionAddForm):
    """An add form for move-to-folder actions.
    """
    schema = ICopyAction
    label = _(u'Add Copy Action')
    description = _(u'A copy action can copy an object to a different folder.')
    Type = CopyAction


class CopyAddFormView(ContentRuleFormWrapper):
    form = CopyAddForm


class CopyEditForm(ActionEditForm):
    """An edit form for copy rule actions.

    z3c.form does all the magic here.
    """
    schema = ICopyAction
    label = _(u'Edit Copy Action')
    description = _(u'A copy action can copy an object to a different folder.')
    form_name = _(u'Configure element')


class CopyEditFormView(ContentRuleFormWrapper):
    form = CopyEditForm



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from BTrees.OOBTree import OOSet
from OFS.SimpleItem import SimpleItem
from plone.contentrules.engine.interfaces import IRuleAssignmentManager
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.rule import Rule as BaseRule
from Products.CMFCore.interfaces import ISiteRoot
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component import queryUtility


ANNOTATION_KEY = 'plone.app.contentrules.ruleassignments'


class Rule(SimpleItem, BaseRule):
    """A Zope 2 version of a rule, subject to acqusition, but otherwise
    identical.
    """

    __name__ = u''

    @property
    def id(self):
        return '++rule++{0}'.format(self.__name__)


def get_assignments(rule):
    annotations = IAnnotations(rule)
    # do not use setdefault here as it'll write to the database on read
    return annotations.get(ANNOTATION_KEY, OOSet())


def insert_assignment(rule, path):
    annotations = IAnnotations(rule)
    if ANNOTATION_KEY not in annotations:
        annotations[ANNOTATION_KEY] = OOSet()
    annotations[ANNOTATION_KEY].insert(path)


# Events that keep track of rule-to-assignment mappings
def rule_removed(rule, event):

    storage = queryUtility(IRuleStorage)
    rule_name = rule.__name__

    if storage is None:
        return

    portal = getUtility(ISiteRoot)
    for path in get_assignments(rule):
        container = portal.unrestrictedTraverse(path)
        if container is not None:
            assignable = IRuleAssignmentManager(container, None)
            if assignable is not None and rule_name in assignable:
                del assignable[rule_name]


def container_moved(container, event):

    if event.oldParent is None or event.newParent is None or \
            event.oldName is None:
        return

    assignable = IRuleAssignmentManager(container, None)
    storage = queryUtility(IRuleStorage)

    if assignable is None or storage is None:
        return

    old_path = '{0}/{1}'.format(
        '/'.join(event.oldParent.getPhysicalPath()),
        event.oldName,
    )
    new_path = '/'.join(container.getPhysicalPath())

    if aq_base(event.object) is not aq_base(container):
        new_path_of_moved = '/'.join(event.object.getPhysicalPath())
        old_path = old_path + new_path[len(new_path_of_moved):]

    for rule_name in assignable.keys():
        rule = storage.get(rule_name, None)
        if rule is not None:
            assignments = get_assignments(rule)
            if old_path in assignments:
                assignments.remove(old_path)
                assignments.insert(new_path)


def container_removed(container, event):

    assignable = IRuleAssignmentManager(container, None)
    storage = queryUtility(IRuleStorage)

    if assignable is None or storage is None:
        return

    path = '/'.join(container.getPhysicalPath())
    for rule_name in assignable.keys():
        rule = storage.get(rule_name, None)
        if rule is not None:
            assignments = get_assignments(rule)
            if path in assignments:
                assignments.remove(path)



# -*- coding: utf-8 -*-
from plone.app.contentrules import PloneMessageFactory as _
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.browser.navigation import PhysicalNavigationBreadcrumbs


class RuleBreadcrumbs(PhysicalNavigationBreadcrumbs):

    def breadcrumbs(self):
        portal_url = getToolByName(self.context, 'portal_url')()
        return (
            {
                'absolute_url': '{0}/@@rules-controlpanel'.format(portal_url),
                'Title': _(
                    'title_manage_contentrules',
                    default=u'Content Rules',
                ),
            },
            {
                'absolute_url': '{0}/@@manage-elements'.format(
                    self.context.absolute_url(),
                ),
                'Title': self.context.title or self.context.id,
            },
        )



# -*- coding: utf-8 -*-
from plone.app.contentrules.conditions.role import RoleCondition
from plone.app.contentrules.conditions.role import RoleEditFormView
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleCondition
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.interfaces import IObjectEvent
from zope.interface import implementer


@implementer(IObjectEvent)
class DummyEvent(object):

    def __init__(self, obj):
        self.object = obj


class TestRoleCondition(ContentRulesTestCase):

    def testRegistered(self):
        element = getUtility(IRuleCondition, name='plone.conditions.Role')
        self.assertEqual('plone.conditions.Role', element.addview)
        self.assertEqual('edit', element.editview)
        self.assertEqual(None, element.for_)
        self.assertEqual(None, element.event)

    def testInvokeAddView(self):
        element = getUtility(IRuleCondition, name='plone.conditions.Role')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter(
            (rule, self.portal.REQUEST), name='+condition')
        addview = getMultiAdapter(
            (adding, self.portal.REQUEST), name=element.addview)

        addview.form_instance.update()
        content = addview.form_instance.create(
            data={'role_names': ['Manager', 'Member']})
        addview.form_instance.add(content)

        e = rule.conditions[0]
        self.assertTrue(isinstance(e, RoleCondition))
        self.assertEqual(['Manager', 'Member'], e.role_names)

    def testInvokeEditView(self):
        element = getUtility(IRuleCondition, name='plone.conditions.Role')
        e = RoleCondition()
        editview = getMultiAdapter(
            (e, self.folder.REQUEST), name=element.editview)
        self.assertTrue(isinstance(editview, RoleEditFormView))

    def testExecute(self):
        e = RoleCondition()
        e.role_names = ['Manager', 'Member']

        ex = getMultiAdapter(
            (self.portal, e, DummyEvent(self.folder)), IExecutable)
        self.assertTrue(ex())

        e.role_names = ['Reviewer']

        ex = getMultiAdapter(
            (self.portal, e, DummyEvent(self.portal)), IExecutable)
        self.assertFalse(ex())



# -*- coding: utf-8 -*-
from .dummy import DummyAction
from .dummy import DummyCondition
from plone.app.contentrules.browser.rule import RuleEditFormView
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.contentrules.engine.interfaces import IRuleStorage
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.lifecycleevent.interfaces import IObjectModifiedEvent


class DummyModifiedRule(Rule):

    title = 'My test rule'
    description = 'Test my rule'
    event = IObjectModifiedEvent
    enabled = True


class TestRuleManagementViews(ContentRulesTestCase):

    def testRuleAdding(self):
        adding = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='+rule')
        storage = getUtility(IRuleStorage)
        self.assertEqual(0, len(storage))
        r = Rule()
        adding.add(r)
        self.assertEqual(1, len(storage))
        self.assertTrue(list(storage.values())[0] is r)

    def testRuleAddView(self):
        adding = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='+rule')
        addview = getMultiAdapter(
            (adding, self.portal.REQUEST), name='plone.ContentRule')
        storage = getUtility(IRuleStorage)
        self.assertEqual(0, len(storage))
        addview.form_instance.update()
        content = addview.form_instance.create({'title': 'foo', 'description': 'bar', 'event': None})  # noqa
        addview.form_instance.add(content)
        self.assertEqual(1, len(storage))
        self.assertEqual('foo', list(storage.values())[0].title)

    def testRuleEditView(self):
        r = Rule()
        editview = getMultiAdapter((r, self.portal.REQUEST), name='edit')
        self.assertTrue(isinstance(editview, RuleEditFormView))


class TestRuleElementManagementViews(ContentRulesTestCase):

    def afterSetUp(self):
        self.setRoles(('Manager', ))

    def testRuleStopModification(self):
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()

        rule = self.portal.restrictedTraverse('++rule++foo')
        view = rule.restrictedTraverse('manage-elements')
        view.template = lambda: 'No template thanks'

        self.portal.REQUEST.form['stopExecuting'] = 'on'
        self.portal.REQUEST.form['form.button.Save'] = True
        self.addAuthToRequest()

        self.assertEqual(False, rule.stop)
        view()
        self.assertEqual(True, rule.stop)

    def testRuleConditionAdding(self):
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')
        adding = getMultiAdapter(
            (rule, self.portal.REQUEST), name='+condition')
        d = DummyCondition()
        self.assertEqual(0, len(rule.conditions))
        adding.add(d)
        self.assertEqual(1, len(rule.conditions))
        self.assertTrue(rule.conditions[0] is d)

    def testRuleActionAdding(self):
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')
        adding = getMultiAdapter((rule, self.portal.REQUEST), name='+action')
        d = DummyAction()
        self.assertEqual(0, len(rule.actions))
        adding.add(d)
        self.assertEqual(1, len(rule.actions))
        self.assertTrue(rule.actions[0] is d)

    def testRulesControlPanel(self):
        portal = self.portal
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = DummyModifiedRule()
        controlpanel = portal.restrictedTraverse('@@rules-controlpanel')
        registered_rules = controlpanel.registeredRules()
        self.assertEqual(1, len(registered_rules))
        registered_rule = registered_rules[0]
        self.assertEqual(
            registered_rule['row_class'],
            'trigger-iobjectmodifiedevent state-enabled assignment-unassigned',
        )
        self.assertEqual(registered_rule['trigger'],
                         'Object modified')
        self.assertTrue(registered_rule['enabled'])
        self.assertFalse(registered_rule['assigned'])

        rule_types = controlpanel.ruleTypesToShow()
        rule_types_ids = [r['id'] for r in rule_types]
        self.assertTrue('trigger-iobjectmodifiedevent' in rule_types_ids)
        self.assertFalse('trigger-iobjectaddedevent' in rule_types_ids)

        rule_states = controlpanel.statesToShow()
        rule_states_ids = [r['id'] for r in rule_states]
        self.assertTrue('state-enabled' in rule_states_ids)
        self.assertTrue('state-disabled' in rule_states_ids)

        # enable rule
        portal.REQUEST['rule-id'] = 'foo'
        self.addAuthToRequest()
        portal.restrictedTraverse('@@contentrule-disable').disable_rule()
        registered_rules = controlpanel.registeredRules()
        self.assertFalse(registered_rules[0]['enabled'])

        portal.restrictedTraverse('@@contentrule-enable').enable_rule()
        registered_rules = controlpanel.registeredRules()
        self.assertTrue(registered_rules[0]['enabled'])

        # works without ajax
        portal.REQUEST.form['rule-id'] = 'foo'
        portal.REQUEST.form['form.button.DisableRule'] = '1'
        portal.restrictedTraverse('@@rules-controlpanel')()
        registered_rules = controlpanel.registeredRules()
        self.assertFalse(registered_rules[0]['enabled'])

        portal.restrictedTraverse('@@contentrule-delete').delete_rule()
        registered_rules = controlpanel.registeredRules()
        self.assertEqual(0, len(registered_rules))

    def testChangeGloballyEnable(self):
        storage = getUtility(IRuleStorage)
        portal = self.portal
        self.addAuthToRequest()

        portal.restrictedTraverse(
            '@@contentrule-globally-enable').globally_enable()
        self.assertTrue(storage.active)

        portal.restrictedTraverse(
            '@@contentrule-globally-disable').globally_disable()
        self.assertFalse(storage.active)

        portal.restrictedTraverse(
            '@@contentrule-globally-enable').globally_enable()
        self.assertTrue(storage.active)

        # without ajax
        self.portal.REQUEST.form



# -*- coding: utf-8 -*-
from plone.app.contentrules import api
from plone.app.contentrules.rule import get_assignments
from plone.app.contentrules.rule import insert_assignment
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.testing import PLONE_APP_CONTENTRULES_FUNCTIONAL_TESTING  # noqa: E501
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.contentrules.engine.assignments import RuleAssignment
from plone.contentrules.engine.interfaces import IRuleAssignmentManager
from plone.contentrules.engine.interfaces import IRuleStorage
from zope.component import getUtility

import transaction
import unittest


class TestRuleAssignmentMapping(unittest.TestCase):

    layer = PLONE_APP_CONTENTRULES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'f1')
        self.folder = self.portal['f1']
        self.folder.invokeFactory('Document', 'd1')
        self.portal.invokeFactory('Folder', 'target')
        self.folder.invokeFactory('Folder', 'f1')
        self.folder.f1.invokeFactory('Folder', 'f11')
        self.folder.f1.invokeFactory('Folder', 'f12')

        self.storage = getUtility(IRuleStorage)
        self.storage['r1'] = Rule()
        self.storage['r2'] = Rule()
        self.storage['r3'] = Rule()

        self.f11a = IRuleAssignmentManager(self.folder.f1.f11)
        self.f11a['r1'] = RuleAssignment('r1', bubbles=True)
        insert_assignment(self.storage['r1'],
                          '/'.join(self.folder.f1.f11.getPhysicalPath()))

        self.f12a = IRuleAssignmentManager(self.folder.f1.f12)
        self.f12a['r1'] = RuleAssignment('r1', bubbles=True)
        insert_assignment(self.storage['r1'],
                          '/'.join(self.folder.f1.f12.getPhysicalPath()))

        self.f12a['r2'] = RuleAssignment('r2', bubbles=True)
        insert_assignment(self.storage['r2'],
                          '/'.join(self.folder.f1.f12.getPhysicalPath()))

    def testRuleRemoved(self):
        self.assertTrue('r1' in self.f11a)
        self.assertTrue('r1' in self.f12a)

        del self.storage['r1']

        self.assertFalse('r1' in self.f11a)
        self.assertFalse('r1' in self.f12a)

    def testContainerMoved(self):
        f12path = '/'.join(self.folder.f1.f12.getPhysicalPath())
        self.assertTrue(f12path in get_assignments(self.storage['r1']))
        self.assertTrue(f12path in get_assignments(self.storage['r2']))

        transaction.savepoint(1)
        self.folder.f1.manage_renameObject('f12', 'f12a')
        f12apath = '/'.join(self.folder.f1.f12a.getPhysicalPath())

        self.assertFalse(f12path in get_assignments(self.storage['r1']))
        self.assertFalse(f12path in get_assignments(self.storage['r2']))

        self.assertTrue(f12apath in get_assignments(self.storage['r1']))
        self.assertTrue(f12apath in get_assignments(self.storage['r1']))

    def testParentOfContainerMoved(self):
        f12path = '/'.join(self.folder.f1.f12.getPhysicalPath())
        self.assertTrue(f12path in get_assignments(self.storage['r1']))
        self.assertTrue(f12path in get_assignments(self.storage['r2']))

        transaction.savepoint(1)
        self.folder.manage_renameObject('f1', 'f1a')
        f12apath = '/'.join(self.folder.f1a.f12.getPhysicalPath())

        self.assertFalse(f12path in get_assignments(self.storage['r1']))
        self.assertFalse(f12path in get_assignments(self.storage['r2']))

        self.assertTrue(f12apath in get_assignments(self.storage['r1']))
        self.assertTrue(f12apath in get_assignments(self.storage['r1']))

    def testContainerRemoved(self):
        f12path = '/'.join(self.folder.f1.f12.getPhysicalPath())
        self.assertTrue(f12path in get_assignments(self.storage['r1']))
        self.assertTrue(f12path in get_assignments(self.storage['r2']))

        transaction.savepoint(1)
        self.folder._delObject('f1')

        self.assertFalse(f12path in get_assignments(self.storage['r1']))
        self.assertFalse(f12path in get_assignments(self.storage['r2']))

    def testRuleAssignmentRemovedAPI(self):
        self.assertTrue('r1' in self.f11a)
        self.assertTrue('r1' in self.f12a)

        api.unassign_rule(self.folder.f1.f11, 'r1')

        self.assertFalse('r1' in self.f11a)
        self.assertTrue('r1' in self.f12a)

    def testRuleAssignmentEditedAPI(self):
        self.assertTrue(self.f11a['r1'].bubbles)
        self.assertTrue(self.f11a['r1'].enabled)

        api.edit_rule_assignment(self.folder.f1.f11, 'r1',
                                 bubbles=False, enabled=False)

        self.assertFalse(self.f11a['r1'].bubbles)
        self.assertFalse(self.f11a['r1'].enabled)

        api.edit_rule_assignment(self.folder.f1.f11, 'r1',
                                 bubbles=True, enabled=True)

        self.assertTrue(self.f11a['r1'].bubbles)
        self.assertTrue(self.f11a['r1'].enabled)

    def testRuleAssignmentAddedAPI(self):
        api.assign_rule(self.folder.f1.f11, 'r2', enabled=True, bubbles=True)
        self.assertTrue('r2' in self.f11a)
        self.assertTrue(self.f11a['r2'].enabled)
        self.assertTrue(self.f11a['r2'].bubbles)

        api.assign_rule(self.folder.f1.f11, 'r3', enabled=True, bubbles=False,
                        insert_before='r2')
        self.assertTrue('r3' in self.f11a)
        self.assertTrue(self.f11a['r3'].enabled)
        self.assertFalse(self.f11a['r3'].bubbles)

        self.assertEqual(self.f11a.keys(), ['r1', 'r3', 'r2'])



# -*- coding: utf-8 -*-
from OFS.SimpleItem import SimpleItem
from plone.app.contentrules import PloneMessageFactory as _
from plone.app.contentrules.browser.formhelper import AddForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.app.contentrules.browser.formhelper import EditForm
from plone.contentrules.rule.interfaces import IExecutable
from plone.contentrules.rule.interfaces import IRuleElementData
from Products.CMFCore.utils import getToolByName
from z3c.form import form
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class IWorkflowStateCondition(Interface):
    """Interface for the configurable aspects of a workflow state condition.

    This is also used to create add and edit forms, below.
    """

    wf_states = schema.Set(
        title=_(u'Workflow state'),
        description=_(u'The workflow states to check for.'),
        required=True,
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.WorkflowStates'
        )
    )


@implementer(IWorkflowStateCondition, IRuleElementData)
class WorkflowStateCondition(SimpleItem):
    """The actual persistent implementation of the workflow state condition
    element.py.
    """

    wf_states = []
    element = 'plone.conditions.WorkflowState'

    @property
    def summary(self):
        return _(
            u'Workflow states are: ${states}',
            mapping=dict(states=', '.join(self.wf_states))
        )


@implementer(IExecutable)
@adapter(Interface, IWorkflowStateCondition, Interface)
class WorkflowStateConditionExecutor(object):
    """The executor for this condition.
    """

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        portal_workflow = getToolByName(self.context, 'portal_workflow', None)
        if portal_workflow is None:
            return False
        state = portal_workflow.getInfoFor(
            self.event.object, 'review_state', None)
        if state is None:
            return False
        return state in self.element.wf_states


class WorkflowStateAddForm(AddForm):
    """An add form for workflow state conditions.
    """
    schema = IWorkflowStateCondition
    label = _(u'Add Workflow State Condition')
    description = _(u'A workflow state condition can restrict rules to '
                    u'objects in particular workflow states')
    form_name = _(u'Configure element')

    def create(self, data):
        c = WorkflowStateCondition()
        form.applyChanges(self, c, data)
        return c


class WorkflowStateAddFormView(ContentRuleFormWrapper):
    form = WorkflowStateAddForm


class WorkflowStateEditForm(EditForm):
    """An edit form for portal type conditions

    z3c.form does all the magic here.
    """
    schema = IWorkflowStateCondition
    label = _(u'Edit Workflow State Condition')
    description = _(u'A workflow state condition can restrict rules to '
                    u'objects in particular workflow states')
    form_name = _(u'Configure element')


class WorkflowStateEditFormView(ContentRuleFormWrapper):
    form = WorkflowStateEditForm
